@@ -27,17 +27,11 @@
 #include "config.h"
 
 #include <gnome.h>
-#include <gdk/gdkkeysyms.h>
-#include <gdk/gdki18n.h>
-
-#include <stdio.h>
-#include <sys/stat.h>
-#include <ctype.h>
+#include <string.h>
 
 /*
  * LibBalsa includes.
  */
-#include "libbalsa.h"
 #include "address-entry.h"
 
 
@@ -45,6 +39,7 @@
  * structure is declared here to keep it opaque */
 struct _LibBalsaAddressEntry {
     GtkEntry parent;
+    gpointer dummy;             /* GtkEntry needs more space? */
 
     GList *active;              /* A GList of email addresses.
                                  * Caution! active may not
@@ -62,7 +57,7 @@ struct _LibBalsaAddressEntry {
 /*
  * Global variable.  We need this for access to parent methods.
  */
-static GtkWidgetClass *parent_class = NULL;
+static GtkEntryClass *parent_class = NULL;
 
 
 /*
@@ -113,8 +108,8 @@ static void libbalsa_delete_backward_word	(LibBalsaAddressEntry *);
 static void libbalsa_delete_line		(LibBalsaAddressEntry *);
 static void libbalsa_delete_to_line_end		(LibBalsaAddressEntry *);
 static void libbalsa_force_expand               (LibBalsaAddressEntry *);
-static void libbalsa_paste_clipboard		(LibBalsaAddressEntry *);
-static void libbalsa_cut_clipboard		(LibBalsaAddressEntry *);
+static void libbalsa_paste_clipboard		(GtkEntry *);
+static void libbalsa_cut_clipboard		(GtkEntry *);
 
 /*
  * Map the Control keys to relevant functions.
@@ -125,7 +120,7 @@ static const LibBalsaTextFun control_keys[26] =
 {
     (LibBalsaTextFun)libbalsa_keystroke_home,		/* a */
     (LibBalsaTextFun)libbalsa_move_backward_character,	/* b */
-    (LibBalsaTextFun)gtk_editable_copy_clipboard,	/* c */
+    NULL,						/* c */
     (LibBalsaTextFun)libbalsa_delete_forward_character,	/* d */
     (LibBalsaTextFun)libbalsa_keystroke_end,		/* e */
     (LibBalsaTextFun)libbalsa_move_forward_character,	/* f */
@@ -144,9 +139,9 @@ static const LibBalsaTextFun control_keys[26] =
     NULL,						/* s */
     NULL,						/* t */
     (LibBalsaTextFun)libbalsa_delete_line,		/* u */
-    (LibBalsaTextFun)libbalsa_paste_clipboard,		/* v */
+    NULL,		                                /* v */
     (LibBalsaTextFun)libbalsa_delete_backward_word,	/* w */
-    (LibBalsaTextFun)libbalsa_cut_clipboard,		/* x */
+    NULL,						/* x */
     NULL,						/* y */
     NULL,						/* z */
 };
@@ -199,19 +194,22 @@ GtkType libbalsa_address_entry_get_type(void)
     static GtkType address_entry_type = 0;
 
     if (!address_entry_type) {
-	static const GtkTypeInfo address_entry_info = {
-	    "LibBalsaAddressEntry",
-	    sizeof(LibBalsaAddressEntry),
+	static const GTypeInfo address_entry_info = {
 	    sizeof(LibBalsaAddressEntryClass),
-	    (GtkClassInitFunc) libbalsa_address_entry_class_init,
-	    (GtkObjectInitFunc) libbalsa_address_entry_init,
-	    /* reserved_1 */ NULL,
-	    /* reserved_2 */ NULL,
-	    (GtkClassInitFunc) NULL,
+            NULL,               /* base_init */
+            NULL,               /* base_finalize */
+	    (GClassInitFunc) libbalsa_address_entry_class_init,
+            NULL,               /* class_finalize */
+            NULL,               /* class_data */
+	    sizeof(LibBalsaAddressEntry),
+            0,                  /* n_preallocs */
+	    (GInstanceInitFunc) libbalsa_address_entry_init
 	};
 
 	address_entry_type =
-	    gtk_type_unique(GTK_TYPE_ENTRY, &address_entry_info);
+            g_type_register_static(GTK_TYPE_ENTRY,
+	                           "LibBalsaAddressEntry",
+                                   &address_entry_info, 0);
     }
 
     return address_entry_type;
@@ -222,9 +220,11 @@ libbalsa_address_entry_class_init(LibBalsaAddressEntryClass *klass)
 {
     GtkWidgetClass *gtk_widget_class;
     GtkObjectClass *object_class;
+    GtkEntryClass  *entry_class;
 
     object_class = GTK_OBJECT_CLASS(klass);
     gtk_widget_class = GTK_WIDGET_CLASS(klass);
+    entry_class = GTK_ENTRY_CLASS(klass);
     parent_class = gtk_type_class(GTK_TYPE_ENTRY);
 
     object_class->destroy = libbalsa_address_entry_destroy;
@@ -233,6 +233,9 @@ libbalsa_address_entry_class_init(LibBalsaAddressEntryClass *klass)
     gtk_widget_class->key_press_event = libbalsa_address_entry_key_press;
     gtk_widget_class->focus_out_event = libbalsa_address_entry_focus_out;
     gtk_widget_class->selection_received = libbalsa_address_entry_received;
+
+    entry_class->cut_clipboard = libbalsa_cut_clipboard;
+    entry_class->paste_clipboard = libbalsa_paste_clipboard;
 }
 
 static void
@@ -270,15 +273,6 @@ libbalsa_address_entry_destroy(GtkObject * object)
     entry = GTK_ENTRY(object);
     address_entry = LIBBALSA_ADDRESS_ENTRY(object);
 
-    /* FIXME: the timer is removed by gtk_entry's finalize method--do we
-     * need to do it as well? 
-     * pawsa: no. They are not ours. */
-    /*
-     * Remove timers.
-     if (entry->timer) gtk_timeout_remove (entry->timer);
-     entry->timer = 0;
-    */
-
     /*
      * Free the address list.
      */
@@ -350,6 +344,7 @@ libbalsa_emailData_new(const gchar * str, gint n)
 
     tmp = g_malloc(sizeof(emailData));
     tmp->user = g_strndup(str, n);
+    tmp->address = NULL;
     tmp->match = NULL;
     tmp->cursor = -1;
     tmp->tabs = 0;
@@ -375,6 +370,7 @@ libbalsa_emailData_set_user(emailData * addy, gchar * str)
 
     g_free(addy->user);
     addy->user = str;
+    /* we DO NOT reset addy->address here. */
 }
 
 
@@ -394,6 +390,9 @@ libbalsa_emailData_free(emailData *addy)
     g_return_if_fail(addy != NULL);
 
     g_free(addy->user);
+    if(addy->address) {
+        g_object_unref(addy->address); addy->address = NULL;
+    }
     g_free(addy->match);
     g_free(addy);
 }
@@ -413,9 +412,7 @@ libbalsa_inputData_free(LibBalsaAddressEntry * address_entry)
 {
     GList *list;
 
-    if (!address_entry)
-        return;
-
+    g_return_if_fail(address_entry);
     list = g_list_first(address_entry->active);
     g_list_foreach(list, (GFunc) libbalsa_emailData_free, NULL);
     g_list_free(list);
@@ -582,6 +579,24 @@ libbalsa_move_backward_word(LibBalsaAddressEntry *address_entry)
     libbalsa_address_entry_show(address_entry);
 }
 
+static void
+lbae_filter(gchar * p)
+{
+    gchar *q = p;
+    g_assert(g_utf8_validate(p, -1, NULL));
+    while (*p) {
+	gunichar c = g_utf8_get_char(p);
+	gchar *r = g_utf8_next_char(p);
+	if (g_unichar_iscntrl(c)) {
+	    *q++ = ' ';
+	    p = r;
+	} else if (q < p) {
+	    while (p < r)
+		*q++ = *p++;
+	} else
+	    p = q = r;
+    }
+}
 
 /*************************************************************
  * libbalsa_fill_input:
@@ -619,6 +634,8 @@ libbalsa_fill_input(LibBalsaAddressEntry *address_entry)
     if (!typed)
 	typed = g_strdup("");
 
+    lbae_filter(typed);
+
     /*
      * Split the input string by comma.
      * str contains a list of e-mail addresses seperated by ','.
@@ -868,6 +885,7 @@ static gint
 libbalsa_address_entry_button_press(GtkWidget * widget, GdkEventButton * event)
 {
     LibBalsaAddressEntry *address_entry;
+    gint retval;
 
     g_return_val_if_fail(widget != NULL, FALSE);
     g_return_val_if_fail(LIBBALSA_IS_ADDRESS_ENTRY(widget), FALSE);
@@ -883,8 +901,15 @@ libbalsa_address_entry_button_press(GtkWidget * widget, GdkEventButton * event)
      * Now that we have done the ONE LINE that we needed to do,
      * let's hand over to the parent method.
      */
-    return
+    retval =
         GTK_WIDGET_CLASS(parent_class)->button_press_event(widget, event);
+
+    /*
+     * Process the text in case it changed (e.g. paste).
+     */
+    libbalsa_fill_input(address_entry);
+
+    return retval;
 }
 
 /*************************************************************
@@ -945,7 +970,7 @@ libbalsa_delete_backward_character(LibBalsaAddressEntry *address_entry)
      * First: Cut the clipboard.
      */
     if (libbalsa_get_selection_bounds(editable, NULL, NULL) && !addy->match) {
-	libbalsa_cut_clipboard(address_entry);
+	libbalsa_cut_clipboard(GTK_ENTRY(address_entry));
 	return;
     }
 
@@ -986,10 +1011,11 @@ libbalsa_delete_backward_character(LibBalsaAddressEntry *address_entry)
      * Normal character needs deleting.
      */
     } else {
-        gchar *p;
-        for (p = addy->user + addy->cursor - 1; *p; ++p)
-            *p = *(p + 1);
-	addy->cursor--;
+        gchar *src= addy->user + addy->cursor, *dest = g_utf8_prev_char(src);
+	int chlen = dest-src;
+        while( (*dest++ = *src++) )
+	    ;
+	addy->cursor += chlen;
 	if (*addy->user == '\0')
 	    libbalsa_force_no_match(addy);
 	else if (address_entry->find_match)
@@ -1026,7 +1052,7 @@ libbalsa_delete_forward_character(LibBalsaAddressEntry *address_entry)
     addy = address_entry->active->data;
     editable = GTK_EDITABLE(address_entry);
     if (libbalsa_get_selection_bounds(editable, NULL, NULL) && !addy->match) {
-	libbalsa_cut_clipboard(address_entry);
+	libbalsa_cut_clipboard(GTK_ENTRY(address_entry));
 	return;
     }
 
@@ -1062,9 +1088,9 @@ libbalsa_delete_forward_character(LibBalsaAddressEntry *address_entry)
      * Normal character needs deleting.
      */
     } else {
-        gchar *p;
-        for (p = addy->user + addy->cursor; *p; ++p)
-            *p = *(p + 1);
+        gchar *dest= addy->user + addy->cursor, *src = g_utf8_next_char(dest);
+        while( (*dest++ = *src++) )
+	    ;
     }
 }
 
@@ -1219,7 +1245,7 @@ libbalsa_move_backward_character(LibBalsaAddressEntry *address_entry)
 
     addy = address_entry->active->data;
     if (addy->cursor > 0) {
-	addy->cursor--;
+	addy->cursor = g_utf8_prev_char(addy->user + addy->cursor)-addy->user;
 	libbalsa_force_no_match(addy);
     } else if (g_list_previous(address_entry->active)) {
 	address_entry->active = g_list_previous(address_entry->active);
@@ -1248,7 +1274,7 @@ libbalsa_move_forward_character(LibBalsaAddressEntry *address_entry)
 
     addy = address_entry->active->data;
     if (addy->user[addy->cursor]) {
-	addy->cursor++;
+	addy->cursor = g_utf8_next_char(addy->user + addy->cursor)-addy->user;
 	libbalsa_force_no_match(addy);
     } else if ((list = g_list_next(address_entry->active))) {
 	address_entry->active = list;
@@ -1392,7 +1418,7 @@ libbalsa_keystroke_add_key(LibBalsaAddressEntry *address_entry, gchar *add)
     editable = GTK_EDITABLE(address_entry);
     addy = address_entry->active->data;
     if (libbalsa_get_selection_bounds(editable, NULL, NULL) && !addy->match) {
-	libbalsa_cut_clipboard(address_entry);
+	libbalsa_cut_clipboard(GTK_ENTRY(address_entry));
     }
 
     /*
@@ -1418,7 +1444,7 @@ libbalsa_keystroke_add_key(LibBalsaAddressEntry *address_entry, gchar *add)
      */
     libbalsa_emailData_set_user(addy, g_strconcat(left, add, right, NULL));
     g_free(left);
-    addy->cursor++;
+    addy->cursor += strlen(add);
 
     /*
      * Now search for (any) match.
@@ -1515,13 +1541,10 @@ libbalsa_force_expand(LibBalsaAddressEntry *address_entry)
  *     modifies address_entry
  *************************************************************/
 static void
-libbalsa_paste_clipboard(LibBalsaAddressEntry *address_entry)
+libbalsa_paste_clipboard(GtkEntry *entry)
 {
-    GtkEditable *editable;
-
-    editable = GTK_EDITABLE(address_entry);
-    gtk_editable_paste_clipboard(editable);
-    libbalsa_fill_input(address_entry);
+    parent_class->paste_clipboard(entry);
+    libbalsa_fill_input(LIBBALSA_ADDRESS_ENTRY(entry));
 }
 
 
@@ -1538,13 +1561,10 @@ libbalsa_paste_clipboard(LibBalsaAddressEntry *address_entry)
  *     modifies address_entry
  *************************************************************/
 static void
-libbalsa_cut_clipboard(LibBalsaAddressEntry *address_entry)
+libbalsa_cut_clipboard(GtkEntry *entry)
 {
-    GtkEditable *editable;
-
-    editable = GTK_EDITABLE(address_entry);
-    gtk_editable_cut_clipboard(editable);
-    libbalsa_fill_input(address_entry);
+    parent_class->cut_clipboard(entry);
+    libbalsa_fill_input(LIBBALSA_ADDRESS_ENTRY(entry));
 }
 
 
@@ -1622,7 +1642,7 @@ libbalsa_address_entry_key_press(GtkWidget *widget, GdkEventKey *event)
     case GDK_Insert:	/* done */
 	return_val = TRUE;
 	if (event->state & GDK_SHIFT_MASK) {
-	    libbalsa_paste_clipboard(address_entry);
+	    libbalsa_paste_clipboard(entry);
 	} else if (event->state & GDK_CONTROL_MASK)
 	    gtk_editable_copy_clipboard(editable);
 	break;
@@ -1631,7 +1651,7 @@ libbalsa_address_entry_key_press(GtkWidget *widget, GdkEventKey *event)
 	if (event->state & GDK_CONTROL_MASK) /* done */
 	    libbalsa_delete_forward_word(address_entry);
 	else if (event->state & GDK_SHIFT_MASK) {
-	    libbalsa_cut_clipboard(address_entry);
+	    libbalsa_cut_clipboard(entry);
 	} else
 	    libbalsa_delete_forward_character(address_entry);
 	break;
@@ -1774,9 +1794,14 @@ libbalsa_address_entry_show(LibBalsaAddressEntry *address_entry)
 	 list = g_list_next(list)) {
 	addy = (emailData *)list->data;
 	g_assert(addy != NULL);
-        if (list == address_entry->active)
-            cursor = end = show->len + addy->cursor;
-        g_string_append(show, addy->user);
+        if (list == address_entry->active) {
+	    int curbyte = show->len + addy->cursor;
+	    g_string_append(show, addy->user);
+            cursor = end = 
+		g_utf8_pointer_to_offset(show->str,
+					 show->str + curbyte);
+	} else g_string_append(show, addy->user);
+
 	if (addy->match) {
 	    g_string_append(show, " (");
 	    g_string_append(show, addy->match);
@@ -1866,7 +1891,9 @@ libbalsa_address_entry_new(void)
 {
     LibBalsaAddressEntry *entry;
 
-    entry = gtk_type_new(LIBBALSA_TYPE_ADDRESS_ENTRY);
+    entry =
+        LIBBALSA_ADDRESS_ENTRY(g_object_new
+                               (LIBBALSA_TYPE_ADDRESS_ENTRY, NULL));
     return GTK_WIDGET(entry);
 }
 
@@ -1892,8 +1919,8 @@ libbalsa_address_entry_focus_out(GtkWidget *widget, GdkEventFocus *event)
     g_return_val_if_fail(LIBBALSA_IS_ADDRESS_ENTRY(widget), FALSE);
     g_return_val_if_fail(event != NULL, FALSE);
 
-    GTK_WIDGET_UNSET_FLAGS(widget, GTK_HAS_FOCUS);
 #if BALSA_MAJOR == 1
+    GTK_WIDGET_UNSET_FLAGS(widget, GTK_HAS_FOCUS);
     /* not in gtk+-2.0: */
     gtk_widget_draw_focus(widget);
 #endif                          /* BALSA_MAJOR == 1 */
@@ -1907,7 +1934,8 @@ libbalsa_address_entry_focus_out(GtkWidget *widget, GdkEventFocus *event)
 #ifdef USE_XIM
     gdk_im_end ();
 #endif
-    return FALSE;
+    return
+        GTK_WIDGET_CLASS(parent_class)->focus_out_event(widget, event);
 }
 
 
@@ -2049,3 +2077,37 @@ libbalsa_delete_link(GList * list, GList * link)
     return g_list_delete_link(list, link);
 #endif                          /* BALSA_MAJOR == 1 */
 }
+
+/* libbalsa_address_entry_get_list:
+   create list of LibBalsaAddress objects corresponding to the
+   entry content. If possible, references objects from the address books.
+   if not, creates new ones.
+   The objects must be dereferenced later and the list disposed, eg.
+   g_list_foreach(l, g_object_unref, NULL); g_list_free(l);
+*/
+GList*
+libbalsa_address_entry_get_list(LibBalsaAddressEntry *address_entry)
+{
+    GList *l, *res = NULL;
+    
+    /* FIXME: this should not be necessary but is to parse address list
+     * set by eg. gtk_entry_set_text(). We should do it earlier, though. */
+    if(!address_entry->active) 
+        libbalsa_fill_input(address_entry);
+
+    for(l = g_list_first(address_entry->active); l;  l = l->next) {
+        emailData *ed = l->data;
+        if(!ed->user || !*ed->user)
+            continue;
+        if(ed->address) {
+            g_object_ref(ed->address);
+            res = g_list_append(res, ed->address);
+        } else {
+	    LibBalsaAddress *addr =
+		libbalsa_address_new_from_string(ed->user);
+	    if (addr)
+		res = g_list_append(res, addr);
+        }
+    }
+    return res;
+}
