@@ -27,6 +27,7 @@
  */
 
 #include "filter-funcs.h"
+#include "filter-private.h"
 
 /* Conditions */
 
@@ -101,7 +102,7 @@ libbalsa_condition_new(void)
     newc->condition_not = FALSE;
     newc->match.string = NULL;
     newc->user_header = NULL;
-    filter_errno = FILTER_NOERR;
+    filter_errno=FILTER_NOERR;
 
     return newc;
 }                      /* end libbalsa_condition_new() */
@@ -112,9 +113,9 @@ libbalsa_condition_regex_new(void)
     LibBalsaConditionRegex * new_reg;
 
     new_reg = g_new(LibBalsaConditionRegex,1);
-    new_reg->string = NULL;
-    new_reg->compiled = NULL;
-    filter_errno = FILTER_NOERR;
+    new_reg->string=NULL;
+    new_reg->compiled=NULL;
+    filter_errno=FILTER_NOERR;
 
     return new_reg;
 }
@@ -136,18 +137,18 @@ libbalsa_condition_clone(LibBalsaCondition* cnd)
     new_cnd->match_fields  = cnd->match_fields;
     new_cnd->type          = cnd->type;
     if (cnd->user_header)
-	new_cnd->user_header = g_strdup(cnd->user_header);
+	new_cnd->user_header   = g_strdup(cnd->user_header);
     switch (new_cnd->type) {
     case CONDITION_SIMPLE:
-        new_cnd->match.string = g_strdup(cnd->match.string);
+        new_cnd->match.string=g_strdup(cnd->match.string);
         break;
     case CONDITION_REGEX:
-        for (regex = cnd->match.regexs;regex && (filter_errno==FILTER_NOERR);
-             regex = g_slist_next(regex)) {
+        for (regex=cnd->match.regexs;regex && (filter_errno==FILTER_NOERR);
+             regex=g_slist_next(regex)) {
             new_reg = libbalsa_condition_regex_new();
             new_reg->string = 
                 g_strdup(((LibBalsaConditionRegex*)regex->data)->string);
-            new_cnd->match.regexs =
+            new_cnd->match.regexs = 
                 g_slist_prepend(new_cnd->match.regexs, new_reg);
         }
         new_cnd->match.regexs = g_slist_reverse(new_cnd->match.regexs);
@@ -157,7 +158,7 @@ libbalsa_condition_clone(LibBalsaCondition* cnd)
         new_cnd->match.interval.date_high = cnd->match.interval.date_high;
         break;
     case CONDITION_FLAG:
-        new_cnd->match.flags = cnd->match.flags;
+        new_cnd->match.flags=cnd->match.flags;
     case CONDITION_NONE:
         /* to avoid warnings */
         break;
@@ -168,7 +169,109 @@ libbalsa_condition_clone(LibBalsaCondition* cnd)
     return new_cnd;
 }
 
-/* FIXME : result of certain function of regex compilation are useless
+/* Helper to compare regexs */
+
+static gboolean
+compare_regexs(GSList * c1,GSList * c2)
+{
+    GSList *l2 = g_slist_copy(c2);
+    LibBalsaConditionRegex * r1,* r2;
+
+    for (;c1 && l2;c1 = g_slist_next(c1)) {
+	GSList * tmp;
+	r1 = c1->data;
+	for (tmp = l2;tmp;tmp = g_slist_next(tmp)) {
+	    r2 = tmp->data;
+	    if (g_ascii_strcasecmp(r1->string,r2->string)==0) {
+		l2 = g_slist_remove(l2, r2);
+		break;
+	    }
+	}
+	/* Not found */
+	if (!tmp)  break;
+    }
+    if (!l2 && !c1)
+	return TRUE;
+    g_slist_free(l2);
+    return FALSE;
+}
+
+/* Helper to compare conditions, a bit obscure at first glance
+   but we have to compare complex structure, so we must check
+   all fields.
+*/
+
+gboolean libbalsa_conditions_compare(GSList * cnd1,GSList *cnd2)
+{
+    GSList * l1 = cnd1,* l2 = g_slist_copy(cnd2),* tmp;
+    gboolean OK;
+
+    for (;l1 && l2;l1 = g_slist_next(l1)) {
+	LibBalsaCondition * c1 = l1->data;
+	LibBalsaCondition * c2 = NULL;
+
+	OK = FALSE;
+	for (tmp = l2;tmp && !OK;tmp = g_slist_next(tmp)) {
+	    c2 = tmp->data;
+	    switch (c1->type) {
+	    case CONDITION_SIMPLE:
+		if ((c2->type == CONDITION_SIMPLE) 
+		    && (g_ascii_strcasecmp(c1->match.string,c2->match.string) == 0))
+		    OK = TRUE;
+		break;
+	    
+	    case CONDITION_REGEX:
+		if ((c2->type == CONDITION_REGEX) &&
+		    compare_regexs(c1->match.regexs,c2->match.regexs))
+		    OK = TRUE;
+		break;
+	    case CONDITION_DATE:
+		if ((c2->type == CONDITION_DATE) &&
+		    (c1->match.interval.date_low == c2->match.interval.date_low) &&
+		    (c1->match.interval.date_high == c2->match.interval.date_high))
+		    OK = TRUE;
+		break;
+		
+	    case CONDITION_FLAG:
+		if ((c2->type == CONDITION_FLAG) &&
+		    (c1->match.flags == c2->match.flags))
+		    OK = TRUE;
+		break;
+	    case CONDITION_NONE:
+		break;
+	    }
+	    /* If OK == TRUE conditions are equal until now, let's see if the
+	       equality is complete, comparing the match fields */
+	    OK = OK
+		&& !(CONDITION_CHKMATCH(c1, CONDITION_MATCH_FROM)
+		     ^ CONDITION_CHKMATCH(c2, CONDITION_MATCH_FROM))
+		&& !(CONDITION_CHKMATCH(c1, CONDITION_MATCH_TO)
+		     ^ CONDITION_CHKMATCH(c2, CONDITION_MATCH_TO))
+		&& !(CONDITION_CHKMATCH(c1, CONDITION_MATCH_CC)
+		     ^ CONDITION_CHKMATCH(c2, CONDITION_MATCH_CC))
+		&& !(CONDITION_CHKMATCH(c1, CONDITION_MATCH_SUBJECT)
+		     ^ CONDITION_CHKMATCH(c2, CONDITION_MATCH_SUBJECT))
+		&& !(CONDITION_CHKMATCH(c1, CONDITION_MATCH_BODY)
+		     ^ CONDITION_CHKMATCH(c2, CONDITION_MATCH_BODY));
+	    /* Special case for user headers */
+	    if (OK && CONDITION_CHKMATCH(c1, CONDITION_MATCH_US_HEAD))
+		OK = CONDITION_CHKMATCH(c2, CONDITION_MATCH_US_HEAD)
+		    && c1->user_header && c2->user_header
+		    && g_ascii_strcasecmp(c1->user_header,c2->user_header)==0;
+	}
+	/* We found the current condition of l1, remove it from l2 */
+	if (OK)
+	    l2 = g_slist_remove(l2,c2);
+	/* Else the conditions list differ, stop the process */
+	else break;
+    }
+    /* There is equality only when l1 = l2 = NULL */
+    if (!l2 && !l1)
+	return TRUE;
+    g_slist_free(l2);    
+    return FALSE;
+}
+/* BIG FIXME : result of certain function of regex compilation are useless
  * and we should have a way to tell which regexs we were unable to compile
  * that's for later
  */
@@ -218,9 +321,9 @@ libbalsa_condition_compile_regexs(LibBalsaCondition* cond)
     GSList * regex;
 
     if (cond->type==CONDITION_REGEX)
-	for(regex = cond->match.regexs;
+	for(regex=cond->match.regexs;
             regex && condition_regcomp((LibBalsaConditionRegex*)regex->data);
-            regex = g_slist_next(regex));
+            regex=g_slist_next(regex));
 }                       /* end of condition_compile_regexs */
 
 /* Filters */
@@ -236,7 +339,8 @@ static void
 filter_condition_validity(LibBalsaFilter* fil, LibBalsaCondition* cond)
 {
     /* Test validity of condition */
-    if (CONDITION_CHKMATCH(cond,CONDITION_MATCH_US_HEAD) && (!cond->user_header || cond->user_header[0]=='\0')) {
+    if (CONDITION_CHKMATCH(cond,CONDITION_MATCH_US_HEAD) && 
+	(!cond->user_header || cond->user_header[0]=='\0')) {
 	FILTER_CLRFLAG(fil,FILTER_VALID);
 	return;
     }
@@ -260,15 +364,15 @@ filter_condition_validity(LibBalsaFilter* fil, LibBalsaCondition* cond)
 void
 libbalsa_filter_append_condition(LibBalsaFilter* fil, LibBalsaCondition* cond)
 {
-    filter_condition_validity(fil, cond);
-    fil->conditions = g_slist_append(fil->conditions, cond);
+    filter_condition_validity(fil,cond);
+    fil->conditions=g_slist_append(fil->conditions,cond);
 }
 
 void
 libbalsa_filter_prepend_condition(LibBalsaFilter* fil, LibBalsaCondition* cond)
 {
-    filter_condition_validity(fil, cond);
-    fil->conditions = g_slist_prepend(fil->conditions, cond);
+    filter_condition_validity(fil,cond);
+    fil->conditions=g_slist_prepend(fil->conditions,cond);
 }
 
 /*
@@ -289,8 +393,7 @@ libbalsa_filter_compile_regexs(LibBalsaFilter* fil)
 
     if (fil->conditions) {
 	GSList * lst;
-	for (lst = fil->conditions;lst && filter_errno==FILTER_NOERR;
-	     lst = g_slist_next(lst))
+	for (lst=fil->conditions;lst && filter_errno==FILTER_NOERR;lst=g_slist_next(lst))
 	    libbalsa_condition_compile_regexs((LibBalsaCondition*) lst->data);
 	if (filter_errno != FILTER_NOERR) {
 	    gchar * errorstring =
@@ -370,7 +473,7 @@ libbalsa_filter_new(void)
 
     newfil = g_new(LibBalsaFilter,1);
 
-    newfil->name = NULL;
+    newfil->name=NULL;
     newfil->flags = FILTER_EMPTY;    /* In particular filter is INVALID */
     newfil->conditions = NULL;
     newfil->sound = NULL;
@@ -379,48 +482,48 @@ libbalsa_filter_new(void)
     newfil->action_string = NULL;
     newfil->matching_messages=NULL;
 
-    filter_errno = FILTER_NOERR;
-    return newfil;
+    filter_errno=FILTER_NOERR;
+    return (newfil);
 }				/* end filter_new() */
 
 static GString*
 match_field_decode(LibBalsaCondition* cnd,GString * buffer,gchar * str_format)
 {
-    GString * str = g_string_new("[");
-    gboolean coma = FALSE;
+    GString * str=g_string_new("[");
+    gboolean coma=FALSE;
 
     /* FIXME : what to do with body match ? */
 
-    if (CONDITION_CHKMATCH(cnd, CONDITION_MATCH_TO)) {
-	str = g_string_append(str,"\"To\"");
-	coma = TRUE;
+    if (CONDITION_CHKMATCH(cnd,CONDITION_MATCH_TO)) {
+	str=g_string_append(str,"\"To\"");
+	coma=TRUE;
     }
-    if (CONDITION_CHKMATCH(cnd, CONDITION_MATCH_FROM)) {
-	str = g_string_append(str, coma ? ",\"From\"" : "\"From\"");
-	coma = TRUE;
+    if (CONDITION_CHKMATCH(cnd,CONDITION_MATCH_FROM)) {
+	str=g_string_append(str,coma ? ",\"From\"" : "\"From\"");
+	coma=TRUE;
     }
-    if (CONDITION_CHKMATCH(cnd, CONDITION_MATCH_CC)) {
-	str = g_string_append(str, coma ? ",\"Cc\"" : "\"Cc\"");
-	coma = TRUE;
+    if (CONDITION_CHKMATCH(cnd,CONDITION_MATCH_CC)) {
+	str=g_string_append(str,coma ? ",\"Cc\"" : "\"Cc\"");
+	coma=TRUE;
     }
-    if (CONDITION_CHKMATCH(cnd, CONDITION_MATCH_SUBJECT)) {
-	str = g_string_append(str, coma ? ",\"Subject\"" : "\"Subject\"");
-	coma = TRUE;
+    if (CONDITION_CHKMATCH(cnd,CONDITION_MATCH_SUBJECT)) {
+	str=g_string_append(str,coma ? ",\"Subject\"" : "\"Subject\"");
+	coma=TRUE;
     }
-    if (CONDITION_CHKMATCH(cnd, CONDITION_MATCH_US_HEAD) && cnd->user_header) {
+    if (CONDITION_CHKMATCH(cnd,CONDITION_MATCH_US_HEAD) && cnd->user_header) {
 	if (coma)
-	    str = g_string_append_c(str, ',');
-	str = g_string_append_c(str, '\"');
-	str = g_string_append(str, cnd->user_header);
-	str = g_string_append_c(str, '\"');
+	    str=g_string_append_c(str,',');
+	str=g_string_append_c(str,'\"');
+	str=g_string_append(str,cnd->user_header);
+	str=g_string_append_c(str,'\"');
     }
-    g_string_append(str, "] ");
+    g_string_append(str,"] ");
     if (str->len>3) {
-	gchar * temp = g_strdup_printf(str_format, "header", str->str);
-	buffer = g_string_append(buffer,temp);
+	gchar * temp=g_strdup_printf(str_format,"header",str->str);
+	buffer=g_string_append(buffer,temp);
 	g_free(temp);
     }
-    g_string_free(str, TRUE);
+    g_string_free(str,TRUE);
     return buffer;
 }
 
@@ -432,40 +535,40 @@ export_condition(LibBalsaCondition* cnd, GString * buffer)
     gboolean parent;
 
     if (cnd->condition_not)
-	buffer = g_string_append(buffer,"not ");
+	buffer=g_string_append(buffer,"not ");
 
     switch (cnd->type) {
     case CONDITION_SIMPLE:
-	str = g_strconcat("%s :contains %s ", "\"", cnd->match.string, "\"", NULL);
-	buffer = match_field_decode(cnd,buffer,str);
+	str=g_strconcat("%s :contains %s ","\"",cnd->match.string,"\"",NULL);
+	buffer=match_field_decode(cnd,buffer,str);
 	g_free(str);
 	break;
     case CONDITION_REGEX:
 	if (cnd->match.regexs->next) {
-	    buffer = g_string_append(buffer,"ANYOF(");
-	    parent = TRUE;
+	    buffer=g_string_append(buffer,"ANYOF(");
+	    parent=TRUE;
 	}
-	else parent = FALSE;
-	for (lst = cnd->match.regexs;lst;) {
+	else parent=FALSE;
+	for (lst=cnd->match.regexs;lst;) {
 	    /* This is not in RFC 3028, consider it as an extension :) */
-	    str = g_strconcat("%s :regex %s ",
-			      ((LibBalsaConditionRegex*)lst->data)->string, NULL);
-	    buffer = match_field_decode(cnd, buffer, str);
+	    str=g_strconcat("%s :regex %s ",
+                            ((LibBalsaConditionRegex*)lst->data)->string,NULL);
+	    buffer=match_field_decode(cnd, buffer, str);
 	    g_free(str);
-	    lst = g_slist_next(lst);
+	    lst=g_slist_next(lst);
 	    if (lst)
-		buffer = g_string_append_c(buffer, ',');
+		buffer=g_string_append_c(buffer,',');
 	}
-	if (parent) buffer = g_string_append_c(buffer, ')');
+	if (parent) buffer = g_string_append_c(buffer,')');
 	break;
     case CONDITION_FLAG:
 	/* FIXME */
-	buffer = g_string_append(buffer, "TRUE");
+	buffer=g_string_append(buffer,"TRUE");
     case CONDITION_DATE:
 	/*FIXME */
     case CONDITION_NONE:
 	/* Should not occur */
-	buffer = g_string_append(buffer, "TRUE");
+	buffer=g_string_append(buffer,"TRUE");
     }
     return buffer;
 }
@@ -480,52 +583,129 @@ libbalsa_filter_export_sieve(LibBalsaFilter* fil, gchar* filename)
     gchar * temp;
     gboolean parent;
 
-    fp = fopen(filename,"w");
+    fp=fopen(filename,"w");
     if (!fp) return FALSE;
-    buffer =
-	g_string_new("# Sieve script automatically generated by Balsa"
-		     " (Exporting a Balsa filter)\n");
+    buffer=g_string_new("# Sieve script automatically generated by Balsa (Exporting a Balsa filter)\n");
     if (fil->conditions) {
-	buffer = g_string_append(buffer, "IF ");
+	buffer=g_string_append(buffer,"IF ");
 	if (fil->conditions->next) {
 	    if (fil->conditions_op==FILTER_OP_OR)
-		buffer = g_string_append(buffer, "ANYOF(");    
-	    else buffer = g_string_append(buffer, "ALLOF(");
-	    parent = TRUE;
+		buffer=g_string_append(buffer,"ANYOF(");    
+	    else buffer=g_string_append(buffer,"ALLOF(");
+	    parent=TRUE;
 	}
-	else parent = FALSE;
-	for (conds = fil->conditions;conds;) {
-	    LibBalsaCondition* cnd = (LibBalsaCondition*)conds->data;
-	    buffer = export_condition(cnd, buffer);
-	    conds = g_slist_next(conds);
+	else parent=FALSE;
+	for (conds=fil->conditions;conds;) {
+	    LibBalsaCondition* cnd=(LibBalsaCondition*)conds->data;
+	    buffer=export_condition(cnd,buffer);
+	    conds=g_slist_next(conds);
 	    if (conds)
-		buffer = g_string_append(buffer, ",\n");
+		buffer=g_string_append(buffer,",\n");
 	}
 	if (parent) 
-	    buffer = g_string_append(buffer, ")\n{\n");
+	    buffer=g_string_append(buffer,")\n{\n");
 	else
-	    buffer = g_string_append(buffer,"\n{\n");
+	    buffer=g_string_append(buffer,"\n{\n");
 	switch (fil->action) {
 	case FILTER_COPY:
 	    /* FIXME : I translate COPY to a keep;fileinto sequence, don't know if it's OK */
-	    buffer = g_string_append(buffer, "keep;\n");
+	    buffer=g_string_append(buffer,"keep;\n");
 	case FILTER_MOVE:
-	    temp = g_strconcat("fileinto \"", fil->action_string, "\";\n", NULL);
-	    buffer = g_string_append(buffer, temp);
+	    temp=g_strconcat("fileinto \"",fil->action_string,"\";\n",NULL);
+	    buffer=g_string_append(buffer,temp);
 	    g_free(temp);
 	    break;
 	case FILTER_TRASH:
-	    buffer = g_string_append(buffer, "discard;\n");
+	    buffer=g_string_append(buffer,"discard;\n");
 	    break;
 	    /* FIXME how to code other actions */
         case FILTER_NOTHING: break;
         case FILTER_PRINT:   break;
+        case FILTER_CHFLAG:  break;
         case FILTER_RUN:     break;
 	}
-	buffer = g_string_append(buffer, "}\n");
+	buffer=g_string_append(buffer,"}\n");
     }
-    nb = fwrite(buffer->str, buffer->len, 1, fp);
-    g_string_free(buffer, TRUE);
-    if (fclose(fp)!=0) nb = 0;
+    nb=fwrite(buffer->str,buffer->len,1,fp);
+    g_string_free(buffer,TRUE);
+    if (fclose(fp)!=0) nb=0;
     return nb==1 ? TRUE : FALSE;
 }             /* end of filter_export_sieve */
+
+/*
+ * compare_filters
+ *
+ * callback for sorting a GtkTreeView
+ */
+static gint
+compare_filters(GtkTreeModel * model, GtkTreeIter * a, GtkTreeIter * b,
+                gpointer user_data)
+{
+    gchar *stra, *strb;
+    gint ret_val;
+
+    gtk_tree_model_get(model, a, 0, &stra, -1);
+    gtk_tree_model_get(model, b, 0, &strb, -1);
+
+    ret_val = g_ascii_strcasecmp(stra, strb);
+
+    g_free(stra);
+    g_free(strb);
+
+    return ret_val;
+}
+
+/*
+ * libbalsa_filter_list_new
+ *
+ * create a GtkTreeView
+ *
+ * with_data            does the underlying store need a data column?
+ * title                for the string column--if NULL, the header is
+ *                      invisible
+ * mode                 selection mode
+ * selection_changed_cb callback for the "changed" signal of the
+ *                      associated GtkTreeSelection
+ * sorted               does the list need to be sorted?
+ */
+GtkTreeView *
+libbalsa_filter_list_new(gboolean with_data, const gchar * title,
+                         GtkSelectionMode mode,
+                         GCallback selection_changed_cb, gboolean sorted)
+{
+    GtkListStore *list_store;
+    GtkTreeView *view;
+    GtkTreeSelection *selection;
+    GtkCellRenderer *renderer;
+    GtkTreeViewColumn *column;
+
+    list_store = with_data ?
+        gtk_list_store_new(2, G_TYPE_STRING, G_TYPE_POINTER) :
+        gtk_list_store_new(1, G_TYPE_STRING);
+    view = GTK_TREE_VIEW(gtk_tree_view_new_with_model
+                         (GTK_TREE_MODEL(list_store)));
+    g_object_unref(list_store);
+
+    renderer = gtk_cell_renderer_text_new();
+    column =
+        gtk_tree_view_column_new_with_attributes(title, renderer, "text",
+                                                 0, NULL);
+    gtk_tree_view_append_column(view, column);
+    if (!title)
+        gtk_tree_view_set_headers_visible(view, FALSE);
+
+    selection = gtk_tree_view_get_selection(view);
+    gtk_tree_selection_set_mode(selection, mode);
+    if (selection_changed_cb)
+        g_signal_connect(G_OBJECT(selection), "changed",
+                         selection_changed_cb, NULL);
+
+    if (sorted) {
+        gtk_tree_sortable_set_sort_func(GTK_TREE_SORTABLE(list_store), 0,
+                                        compare_filters, NULL, NULL);
+        gtk_tree_sortable_set_sort_column_id(GTK_TREE_SORTABLE(list_store),
+                                             0, GTK_SORT_ASCENDING);
+    }
+
+    return view;
+}                               /* end of libbalsa_filter_list_new */
