@@ -41,7 +41,7 @@
 #include "browser.h"
 
 static void
-libbalsa_scanner_mh_dir(GNode *rnode,
+libbalsa_scanner_mdir(GNode *rnode,
 	       		const gchar * prefix, 
 			LocalHandler folder_handler, 
 			LocalHandler mailbox_handler)
@@ -73,12 +73,14 @@ libbalsa_scanner_mh_dir(GNode *rnode,
 	     * if we think that this looks like a mailbox, include it as such.
 	     * otherwise we'll lose the mail in this folder
 	     */
-	    if (libbalsa_mailbox_type_from_path(filename) == LIBBALSA_TYPE_MAILBOX_MH) {
+	    GType foo = libbalsa_mailbox_type_from_path(filename);
+	    if( (foo == LIBBALSA_TYPE_MAILBOX_MH) ||
+		(foo == LIBBALSA_TYPE_MAILBOX_MAILDIR ) ) {
 		parent_node = mailbox_handler(rnode, de->d_name, filename);
-		libbalsa_scanner_mh_dir(parent_node, filename, 
-					folder_handler, mailbox_handler);
+		libbalsa_scanner_mdir(parent_node, filename, 
+				      folder_handler, mailbox_handler);
 	    }
-	}
+	} 
 	/* ignore regular files */
     }
     closedir(dpc);
@@ -91,7 +93,6 @@ libbalsa_scanner_local_dir(GNode *rnode, const gchar * prefix,
 {
     DIR *dpc;
     struct dirent *de;
-    const gchar * name;
     char filename[PATH_MAX];
     struct stat st;
     GtkType mailbox_type;
@@ -113,17 +114,18 @@ libbalsa_scanner_local_dir(GNode *rnode, const gchar * prefix,
 	if (S_ISDIR(st.st_mode)) {
 	    mailbox_type = libbalsa_mailbox_type_from_path(filename);
 
-	    if (mailbox_type == LIBBALSA_TYPE_MAILBOX_MH) {
+	    if ( (mailbox_type == LIBBALSA_TYPE_MAILBOX_MH) ||
+		 (mailbox_type == LIBBALSA_TYPE_MAILBOX_MAILDIR) ) {
 		current_node = mailbox_handler(rnode, de->d_name, filename);
-		libbalsa_scanner_mh_dir(current_node, filename, 
+		libbalsa_scanner_mdir(current_node, filename, 
 				        folder_handler, mailbox_handler);
-	    } else if (mailbox_type == LIBBALSA_TYPE_MAILBOX_MAILDIR) {
-		mailbox_handler(rnode, de->d_name, filename);
 	    } else {
-		name = g_basename(prefix);
+                gchar *name = g_path_get_basename(prefix);
+
 		current_node = folder_handler(rnode, name, filename);
 		libbalsa_scanner_local_dir(current_node, filename, 
 					   folder_handler, mailbox_handler);
+                g_free(name);
 	    }
 	} else {
 	    mailbox_type = libbalsa_mailbox_type_from_path(filename);
@@ -142,15 +144,62 @@ void imap_add_folder (char delim, char *folder, int noselect, int noinferiors,
 /* executed with GDK lock OFF.
  * see HACKING file for proper locking order description.
  */
+
+/* libbalsa_imap_browse: recursive helper.
+ *
+ * path:                the imap path to be browsed;
+ * browser_state:       a libmutt structure;
+ * server:              the LibBalsa server for the tree;
+ * check_imap_path:     a callback for finding out whether a path must
+ *                      be scanned;
+ * depth:               depth of the recursion.
+ */
+static void
+libbalsa_imap_browse(const gchar * path, struct browser_state *state,
+                     LibBalsaServer * server, ImapCheck check_imap_path,
+                     ImapMark mark_imap_path,
+                     guint * depth)
+{
+    gchar *imap_path;
+    GList *list, *el;
+    gboolean browse;
+
+    state->subfolders = NULL;
+    FREE(&state->folder);
+
+    mark_imap_path(path, state->cb_data);
+    
+    imap_path = libbalsa_imap_path(server, path);
+    imap_browse(imap_path, state);
+    g_free(imap_path);
+
+    list = state->subfolders;
+    state->subfolders = NULL;
+
+    ++*depth;
+    browse = FALSE;
+    for (el = list; el && !browse; el = g_list_next(el))
+        browse = check_imap_path(server, el->data, *depth);
+
+    if (browse)
+        for (el = list; el; el = g_list_next(el))
+            libbalsa_imap_browse(el->data, state, server, check_imap_path,
+                                 mark_imap_path, depth);
+    --*depth;
+
+    g_list_foreach(list, (GFunc) g_free, NULL);
+    g_list_free(list);
+}
+
 void
 libbalsa_scanner_imap_dir(GNode *rnode, LibBalsaServer * server, 
 			  const gchar* path, gboolean subscribed, 
-                          gboolean list_inbox, int depth,
-			  ImapHandler folder_handler, 
-			  ImapHandler mailbox_handler)
+                          gboolean list_inbox,
+                          ImapCheck check_imap_path,
+                          ImapMark mark_imap_path,
+			  ImapHandler mailbox_handler,
+			  gpointer cb_data)
 {
-    gchar* imap_path;
-    GList* list = NULL, *el;
     struct browser_state state;
     int i;
     
@@ -158,7 +207,7 @@ libbalsa_scanner_imap_dir(GNode *rnode, LibBalsaServer * server,
     state.imap_browse = 1;
     state.rnode = rnode;
     state.mailbox_handler = (void(*)())mailbox_handler;
-    state.folder_handler = (void(*)())folder_handler;
+    state.cb_data         = cb_data;
     if(!FileMask.rx) { /* allocate it once, on the first run */
        FileMask.rx = (regex_t *) safe_malloc (sizeof (regex_t));
        if( (i=REGCOMP(FileMask.rx,"!^\\.[^.]",0)) != 0) {
@@ -170,104 +219,60 @@ libbalsa_scanner_imap_dir(GNode *rnode, LibBalsaServer * server,
     }
     /* try getting password, quit on cancel */
     if (!server->passwd) {
-        gdk_threads_enter();
         server->passwd = libbalsa_server_get_password(server, NULL);
-        gdk_threads_leave();
         if(!server->passwd)
             return;
     }
-	
-    gdk_threads_enter(); libbalsa_lock_mutt();
 
-    if (list_inbox)
+    libbalsa_lock_mutt();
+
+    if (list_inbox) {
         /* force INBOX into the mailbox list
          * delim doesn't matter, so we'll give it '/'
-         * and we'll mark it as not scanned, in case it
-         * has any subfolders */
-        mailbox_handler(rnode, "INBOX", '/', FALSE);
+         * and we'll mark it as scanned, because the only reason for
+         * using this option is to pickup an INBOX that isn't in the
+         * tree specified by the prefix */
+        mailbox_handler("INBOX", '/', FALSE, TRUE, cb_data);
+        mark_imap_path("INBOX", cb_data);
+    }
 
-    safe_free((void **)&ImapUser);   ImapUser = safe_strdup(server->user);
-    safe_free((void **)&ImapPass);   ImapPass = safe_strdup(server->passwd);
+    reset_mutt_passwords(server);
     /* subscribed triggers a bug in libmutt, disable it now */
     if(subscribed)
 	set_option(OPTIMAPLSUB);
     else
 	unset_option(OPTIMAPLSUB);
 
-
-    state.subfolders = g_list_append(NULL, g_strdup(path));
     state.folder = NULL;
+    i = 0;
+    libbalsa_imap_browse(path, &state, server, check_imap_path,
+                         mark_imap_path, &i);
 
-    for(i=0; state.subfolders && i<depth; i++) {
-	state.scanned = (i < depth - 1);
-	list = state.subfolders;
-	state.subfolders = NULL;
-	for(el= g_list_first(list); el; el = g_list_next(el)) {
-	    if(*(char*)el->data)
-		imap_path = g_strdup_printf("imap%s://%s/%s/", 
-#ifdef USE_SSL
-					    server->use_ssl ? "s" : "",
-#else
-					    "",
-#endif
-					    server->host, (char*)el->data);
-	    else 
-		imap_path = g_strdup_printf("imap%s://%s/", 
-#ifdef USE_SSL
-					    server->use_ssl ? "s" : "",
-#else
-					    "",
-#endif
-					    server->host);
-	    FREE(&state.folder);
-	    imap_browse ((char*)imap_path,  &state);
-	    g_free(imap_path);
-	}
-	g_list_foreach(list, (GFunc)g_free, NULL);
-	g_list_free(list); 
-    }
-    g_list_foreach((GList*)state.subfolders, (GFunc)g_free, NULL);
-    g_list_free((GList*)state.subfolders);
     state_free (&state);
-    libbalsa_unlock_mutt(); gdk_threads_leave();
+    libbalsa_unlock_mutt(); 
 }
 
 /* this function ovverrides mutt's one.
- * called with GDK unlocked via sequence:
+ * The calling sequence is:
  * libbalsa_scanner_imap_dir->libmutt::imap_browse-> imap_add_folder.
- * See HACKING file for proper locking order description.
  */
-void imap_add_folder (char delim, char *folder, int noselect,
-  int noinferiors, struct browser_state *state, short isparent)
+void
+imap_add_folder(char delim, char *folder, int noselect, int noinferiors,
+		struct browser_state *state, short isparent)
 {
-    int isFolder = 0;
-    int isMailbox = 0;
+    if (isparent)
+	return;
+
+    imap_unmunge_mbox_name(folder);
 
-    imap_unmunge_mbox_name (folder);
-    /* printf("imap_add_folder. delim: '%c', folder: '%s', noselect: %d\n"
-	   "noinferiors: %d, isparent: %d\n", delim, folder, noselect,
-	   noinferiors, isparent);  */
-    if(isparent) return;
-    libbalsa_unlock_mutt();
-    if(!noselect) {
-	libbalsa_information(LIBBALSA_INFORMATION_DEBUG,
-                             "ADDING MAILBOX %s\n", folder);
-	++isMailbox;
-    }
     /* this extra check is needed for subscribed folder handling. 
-	   Read RFC when in doubt. */
-    if(!g_list_find_custom(state->subfolders, folder,
-			   (GCompareFunc)strcmp)) {
-	libbalsa_information(LIBBALSA_INFORMATION_DEBUG,
-                             "ADDING FOLDER  %s\n", folder);
-	    
-	state->subfolders = g_list_append(state->subfolders,
-					  g_strdup(folder));
-	++isFolder;
-    }
-    if (isMailbox)
-	state->mailbox_handler(state->rnode, folder, delim, state->scanned);
-    else if (isFolder)
-	state->folder_handler(state->rnode, folder, delim, state->scanned);
-    libbalsa_lock_mutt();
+     * Read RFC when in doubt. */
+    if (!g_list_find_custom(state->subfolders, folder,
+			    (GCompareFunc) strcmp)
+	&& !noinferiors)
+	state->subfolders =
+	    g_list_append(state->subfolders, g_strdup(folder));
+
+    state->mailbox_handler(folder, delim, noselect, noinferiors,
+			   state->cb_data);
 }
