@@ -22,7 +22,6 @@
 
 #include "config.h"
 
-#include <errno.h>
 #include <glib.h>
 #include <gnome.h>
 
@@ -31,22 +30,19 @@
 #include <sys/utsname.h>
 #include <stdarg.h>
 
-#ifdef HAVE_GNOME_VFS
-# include <libgnomevfs/gnome-vfs-init.h>
-#endif
-
 #if ENABLE_LDAP
 #include <ldap.h>
 #endif
 
 #include "libbalsa.h"
-#include "mailbackend.h"
 #include "misc.h"
+#include "mailbackend.h"
 
 
 #ifdef BALSA_USE_THREADS
 static GMutex *mutt_lock;
 static pthread_t main_thread_id;
+pthread_mutex_t mailbox_lock = PTHREAD_MUTEX_INITIALIZER;
 #endif
 
 #define POP_SERVER "pop"
@@ -62,6 +58,7 @@ int mutt_any_key_to_continue(const char *s);
 void mutt_clear_error(void);
 
 static void libbalsa_mutt_error(const char *fmt, ...);
+static gchar *qualified_hostname(const char *name);
 
 void
 mutt_message(const char *fmt, ...)
@@ -69,7 +66,8 @@ mutt_message(const char *fmt, ...)
     va_list va_args;
 
     va_start(va_args, fmt);
-    libbalsa_information_varg(LIBBALSA_INFORMATION_MESSAGE, fmt, va_args);
+    libbalsa_information_varg(NULL, LIBBALSA_INFORMATION_MESSAGE,
+                              fmt, va_args);
     va_end(va_args);
 }
 
@@ -88,6 +86,7 @@ mutt_perror (const char *s)
   mutt_error ("%s: %s (errno = %d)", s, p ? p : _("unknown error"), errno);
 }
 
+
 int
 mutt_yesorno(const char *msg, int def)
 {
@@ -108,13 +107,10 @@ libbalsa_init(LibBalsaInformationFunc information_callback)
     struct utsname utsname;
     const char *p;
 
-#ifdef	HAVE_GNOME_VFS
-    if (!gnome_vfs_init ()) {
-        fprintf (stderr, "Cannot initialize the GNOME Virtual File System.\n");
-        return;
-    }
-#endif
     Spoolfile = libbalsa_guess_mail_spool();
+    MhFlagged = "flagged";
+    MhReplied = "replied";
+    MhUnseen  = "unseen";
 
 #ifdef BALSA_USE_THREADS
     if (!g_thread_supported()) {
@@ -139,10 +135,7 @@ libbalsa_init(LibBalsaInformationFunc information_callback)
 
     mutt_error = libbalsa_mutt_error;
 
-    if ( Domainname ) 
-	Fqdn = g_strdup_printf("%s.%s", Hostname, Domainname);
-    else
-	Fqdn = g_strdup(Hostname);
+    Fqdn = qualified_hostname(Hostname);
 
     Sendmail = SENDMAIL;
 
@@ -172,9 +165,9 @@ libbalsa_init(LibBalsaInformationFunc information_callback)
     ReplyRegexp.rx = (regex_t *) safe_malloc (sizeof (regex_t));
     REGCOMP(ReplyRegexp.rx,"^(re([\\[0-9\\]+])*|aw):[ \t]*",0);
 
-    Charset = NULL;
-    /* more likely to least likely or something */
-    SendCharset = "us-ascii:iso-8859-1:iso-8859-15:iso-8859-2:iso-8859-9:iso-8859-13:euc-kr:euc-jp:KOI8-R";    
+    Charset = "UTF-8";
+    /* more likely to least likely or something: order *is* relevant!  */
+    SendCharset = "us-ascii:iso-8859-1:iso-8859-15:iso-8859-2:iso-8859-9:iso-8859-13:KOI8-R:iso-8859-5:euc-kr:euc-jp:UTF-8";    
 
     libbalsa_notify_init();
 
@@ -317,7 +310,7 @@ libbalsa_guess_mail_spool(void)
 	for (i = 0; guesses[i] != NULL; i++) {
 	    spool = g_strconcat(guesses[i], env, NULL);
 
-	    if (g_file_exists(spool))
+	    if (g_file_test(spool, G_FILE_TEST_EXISTS))
 		return spool;
 
 	    g_free(spool);
@@ -423,13 +416,14 @@ gchar *libbalsa_guess_ldif_file()
 	".address.ldif",
 	"address-book.ldif",
 	".address-book.ldif",
+	".addressbook.ldif",
 	NULL
     };
 
     for (i = 0; guesses[i] != NULL; i++) {
 	ldif = gnome_util_prepend_user_home(guesses[i]);
 	
-	if (g_file_exists(ldif))
+	if (g_file_test(ldif, G_FILE_TEST_EXISTS))
 	     return ldif;
 	  
 	g_free(ldif);
@@ -472,10 +466,10 @@ libbalsa_mutt_error(const char *fmt, ...)
 
     va_start(va_args, fmt);
     if(*fmt) 
-	libbalsa_information_varg(LIBBALSA_INFORMATION_WARNING, fmt, va_args);
+	libbalsa_information_varg(NULL, LIBBALSA_INFORMATION_WARNING,
+                                  fmt, va_args);
     va_end(va_args);
 }
-
 /* wraper function */
 void 
 libbalsa_mktemp (char *s) {
@@ -505,12 +499,12 @@ ask_cert_real(X509 *cert)
     char *part[] =
         {"/CN=", "/Email=", "/O=", "/OU=", "/L=", "/ST=", "/C="};
     char buf[SHORT_STRING];
-    char *name = NULL, *c;
+    char *name = NULL, *c, *valid_from;
     GtkWidget* dialog, *label;
     unsigned i;
     int ret;
 
-    GString* str = g_string_new(_("This certificate belongs to:\n"));
+    GString* str = g_string_new(_("<b>This certificate belongs to:</b>\n"));
 
     name = X509_NAME_oneline(X509_get_subject_name (cert), buf, sizeof (buf));
     for (i = 0; i < ELEMENTS(part); i++) {
@@ -518,7 +512,7 @@ ask_cert_real(X509 *cert)
         g_string_append_c(str, '\n');
     }
 
-    g_string_append(str, _("\nThis certificate was issued by:\n"));
+    g_string_append(str, _("\n<b>This certificate was issued by:</b>\n"));
     name = X509_NAME_oneline(X509_get_issuer_name(cert), buf, sizeof (buf));
     for (i = 0; i < ELEMENTS(part); i++) {
         g_string_append(str, x509_get_part (name, part[i]));
@@ -527,31 +521,38 @@ ask_cert_real(X509 *cert)
 
     buf[0] = '\0';
     x509_fingerprint (buf, sizeof (buf), cert);
-    c = g_strdup_printf(_("This certificate is valid\n"
+    valid_from = g_strdup(asn1time_to_string(X509_get_notBefore(cert)));
+    c = g_strdup_printf(_("<b>This certificate is valid</b>\n"
                           "from %s\n"
                           "to %s\n"
-                          "Fingerprint: %s"),
-                        asn1time_to_string(X509_get_notBefore(cert)),
+                          "<b>Fingerprint:</b> %s"),
+                        valid_from,
                         asn1time_to_string(X509_get_notAfter(cert)),
                         buf);
     g_string_append(str, c); g_free(c);
+    g_free(valid_from);
 
-    dialog = gnome_dialog_new(_("IMAP TLS certificate"), 
-                              _("Accept Once"),_("Accept&Save"),
-                              _("Reject"), NULL);
+    dialog = gtk_dialog_new_with_buttons(_("IMAP TLS certificate"), NULL,
+                                         GTK_DIALOG_MODAL,
+                                         _("_Accept Once"), 0,
+                                         _("Accept&_Save"), 1,
+                                         _("_Reject"), GTK_RESPONSE_CANCEL, 
+                                         NULL);
+    gtk_window_set_wmclass(GTK_WINDOW(dialog), "tls_cert_dialog", "Balsa");
     label = gtk_label_new(str->str);
     g_string_free(str, TRUE);
-
-    gtk_box_pack_start(GTK_BOX(GNOME_DIALOG(dialog)->vbox),
+    gtk_label_set_use_markup(GTK_LABEL(label), TRUE);
+    gtk_box_pack_start(GTK_BOX(GTK_DIALOG(dialog)->vbox),
                        label, TRUE, TRUE, 1);
     gtk_widget_show(label);
 
-    switch(gnome_dialog_run_and_close(GNOME_DIALOG(dialog))) {
+    switch(gtk_dialog_run(GTK_DIALOG(dialog))) {
     case 0: ret = OP_MAX; break;
     case 1: ret = OP_SAVE; libbalsa_assure_balsa_dir(); break;
-    case 2:
+    case GTK_RESPONSE_CANCEL:
     default: ret = OP_EXIT; break;
     }
+    gtk_widget_destroy(dialog);
     return ret;
 }
 
@@ -577,7 +578,7 @@ ask_cert_idle(gpointer data)
 }
 /* libmutt_ask_for_cert_acceptance:
    executed with GDK UNLOCKED. see mailbox_imap_open() and
-   imap_folder_imap_dir().
+   imap_dir_cb()/imap_folder_imap_dir().
 */
 int
 libmutt_ask_for_cert_acceptance(X509 *cert)
@@ -585,20 +586,20 @@ libmutt_ask_for_cert_acceptance(X509 *cert)
     static pthread_mutex_t ask_cert_lock = PTHREAD_MUTEX_INITIALIZER;
     AskCertData acd;
 
-    if (pthread_self() == libbalsa_get_main_thread()) 
-        return ask_cert_real(cert);
+    if (pthread_self() == libbalsa_get_main_thread())
+	return ask_cert_real(cert);
 
-    libbalsa_unlock_mutt(); gdk_threads_leave();
     pthread_mutex_lock(&ask_cert_lock);
     pthread_cond_init(&acd.cond, NULL);
     acd.cert = cert;
-    gtk_idle_add(ask_cert_idle, &acd);
+    g_idle_add(ask_cert_idle, &acd);
+    libbalsa_unlock_mutt(); 
     pthread_cond_wait(&acd.cond, &ask_cert_lock);
+    libbalsa_lock_mutt();
     
     pthread_cond_destroy(&acd.cond);
     pthread_mutex_unlock(&ask_cert_lock);
     pthread_mutex_destroy(&ask_cert_lock);
-    gdk_threads_enter(); libbalsa_lock_mutt();
     return acd.res;
 }
 #else /* BALSA_USE_THREADS */
