@@ -26,6 +26,7 @@
 typedef struct _LibBalsaMailbox LibBalsaMailbox;
 typedef struct _LibBalsaMailboxRemote LibBalsaMailboxRemote;
 typedef struct _LibBalsaMessage LibBalsaMessage;
+typedef struct _LibBalsaMessageHeaders LibBalsaMessageHeaders;
 typedef struct _LibBalsaMessageBody LibBalsaMessageBody;
 typedef struct _LibBalsaServer LibBalsaServer;
 
@@ -63,13 +64,16 @@ typedef struct header MuttHeader;
 #include "mailbox_maildir.h"
 
 #define ELEMENTS(x) (sizeof (x) / sizeof (x[0]))
+
 /*
  * Initialize the library
  */
 void libbalsa_init(LibBalsaInformationFunc information_callback);
 void libbalsa_set_spool(gchar * spool);
 
-void libbalsa_show_message_source(LibBalsaMessage* msg);
+void libbalsa_show_message_source(LibBalsaMessage * msg,
+                                  const gchar * font,
+                                  gboolean *escape_specials);
 gchar *libbalsa_rot(const gchar * pass);
 
 gchar *libbalsa_guess_email_address(void);
