@@ -23,9 +23,10 @@
 #ifndef __LIBBALSA_PRIVATE_H__
 #define __LIBBALSA_PRIVATE_H__
 
+#include <unistd.h>
+
 #include "libmutt/mutt.h"
 
-LibBalsaAddress *libbalsa_address_new_from_libmutt(ADDRESS * caddr);
 /* private interfaces to avoid the libbalsa API */
 /* address.c: */
 gchar *libbalsa_address_to_gchar_p(LibBalsaAddress * address, gint n);
@@ -34,7 +35,8 @@ gchar *libbalsa_make_string_from_list_p(const GList * the_list);
 
 
 #ifdef BALSA_USE_THREADS
-
+#include <pthread.h>
+extern pthread_mutex_t mailbox_lock;
 /*  #define DEBUG */
 
 #ifdef DEBUG
@@ -124,7 +126,8 @@ do {\
 
 #define CLIENT_CONTEXT(mailbox)          (mailbox->context)
 #define CLIENT_CONTEXT_OPEN(mailbox)     (CLIENT_CONTEXT (mailbox) != NULL)
-#define CLIENT_CONTEXT_CLOSED(mailbox)   (CLIENT_CONTEXT (mailbox) == NULL)
+#define CLIENT_CONTEXT_CLOSED(mailbox) \
+  (CLIENT_CONTEXT (mailbox) == NULL||CLIENT_CONTEXT (mailbox)->hdrs==NULL)
 #define RETURN_IF_CLIENT_CONTEXT_CLOSED(mailbox)\
 do {\
   if (CLIENT_CONTEXT_CLOSED (mailbox))\
