@@ -22,37 +22,27 @@
 
 #include "config.h"
 
-#include <unistd.h>
-
-#ifdef BALSA_USE_THREADS
-#include <pthread.h>
-#endif
-
+#include <stdio.h>
+#include <fcntl.h>
+#include <errno.h>
+#include <gtk/gtk.h>
 #include "libbalsa.h"
 #include "libbalsa_private.h"
 #include "mailbackend.h"
 #include "pop3.h"
 #include "mailbox.h"
-#include <stdio.h>
-#include <fcntl.h>
-#include <errno.h>
 
-#ifdef BALSA_USE_THREADS
-#include "threads.h"
-#else
-/* FIXME: */
-void config_mailbox_update(LibBalsaMailbox* );
-#endif
-
-#include "mailbox-filter.h"
-#include "filter-file.h"
-#include <libgnome/gnome-defs.h> 
 #include <libgnome/gnome-config.h> 
 #include <libgnome/gnome-i18n.h> 
 
+enum {
+    CONFIG_CHANGED,
+    LAST_SIGNAL
+};
 static LibBalsaMailboxClass *parent_class = NULL;
+static guint libbalsa_mailbox_pop3_signals[LAST_SIGNAL];
 
-static void libbalsa_mailbox_pop3_destroy(GtkObject * object);
+static void libbalsa_mailbox_pop3_finalize(GObject * object);
 static void libbalsa_mailbox_pop3_class_init(LibBalsaMailboxPop3Class *
 					     klass);
 static void libbalsa_mailbox_pop3_init(LibBalsaMailboxPop3 * mailbox);
@@ -65,27 +55,30 @@ static void libbalsa_mailbox_pop3_save_config(LibBalsaMailbox * mailbox,
 static void libbalsa_mailbox_pop3_load_config(LibBalsaMailbox * mailbox,
 					      const gchar * prefix);
 
-static void progress_cb(char *msg, int prog, int tot);
+static void progress_cb(void* m, char *msg, int prog, int tot);
 
-GtkType libbalsa_mailbox_pop3_get_type(void)
+GType
+libbalsa_mailbox_pop3_get_type(void)
 {
-    static GtkType mailbox_type = 0;
+    static GType mailbox_type = 0;
 
     if (!mailbox_type) {
-	static const GtkTypeInfo mailbox_info = {
-	    "LibBalsaMailboxPOP3",
-	    sizeof(LibBalsaMailboxPop3),
+	static const GTypeInfo mailbox_info = {
 	    sizeof(LibBalsaMailboxPop3Class),
-	    (GtkClassInitFunc) libbalsa_mailbox_pop3_class_init,
-	    (GtkObjectInitFunc) libbalsa_mailbox_pop3_init,
-	    /* reserved_1 */ NULL,
-	    /* reserved_2 */ NULL,
-	    (GtkClassInitFunc) NULL,
+            NULL,               /* base_init */
+            NULL,               /* base_finalize */
+	    (GClassInitFunc) libbalsa_mailbox_pop3_class_init,
+            NULL,               /* class_finalize */
+            NULL,               /* class_data */
+	    sizeof(LibBalsaMailboxPop3),
+            0,                  /* n_preallocs */
+	    (GInstanceInitFunc) libbalsa_mailbox_pop3_init
 	};
 
 	mailbox_type =
-	    gtk_type_unique(libbalsa_mailbox_remote_get_type(),
-			    &mailbox_info);
+	    g_type_register_static(LIBBALSA_TYPE_MAILBOX_REMOTE,
+                                   "LibBalsaMailboxPOP3",
+			           &mailbox_info, 0);
     }
 
     return mailbox_type;
@@ -94,15 +87,23 @@ GtkType libbalsa_mailbox_pop3_get_type(void)
 static void
 libbalsa_mailbox_pop3_class_init(LibBalsaMailboxPop3Class * klass)
 {
-    GtkObjectClass *object_class;
+    GObjectClass *object_class;
     LibBalsaMailboxClass *libbalsa_mailbox_class;
 
-    object_class = GTK_OBJECT_CLASS(klass);
+    object_class = G_OBJECT_CLASS(klass);
     libbalsa_mailbox_class = LIBBALSA_MAILBOX_CLASS(klass);
 
-    parent_class = gtk_type_class(libbalsa_mailbox_remote_get_type());
+    parent_class = g_type_class_peek_parent(klass);
+    libbalsa_mailbox_pop3_signals[CONFIG_CHANGED] =
+	g_signal_new("config-changed",
+                     G_TYPE_FROM_CLASS(object_class),
+                     G_SIGNAL_RUN_LAST | G_SIGNAL_NO_HOOKS,
+                     G_STRUCT_OFFSET(LibBalsaMailboxPop3Class,
+                                     config_changed),
+                     NULL, NULL,
+                     g_cclosure_marshal_VOID__VOID, G_TYPE_NONE, 0);
 
-    object_class->destroy = libbalsa_mailbox_pop3_destroy;
+    object_class->finalize = libbalsa_mailbox_pop3_finalize;
 
     libbalsa_mailbox_class->open_mailbox = libbalsa_mailbox_pop3_open;
     libbalsa_mailbox_class->check = libbalsa_mailbox_pop3_check;
@@ -122,13 +123,15 @@ libbalsa_mailbox_pop3_init(LibBalsaMailboxPop3 * mailbox)
     mailbox->delete_from_server = FALSE;
     mailbox->inbox = NULL;
 
+    mailbox->filter = FALSE;
+    mailbox->filter_cmd = NULL;
     remote = LIBBALSA_MAILBOX_REMOTE(mailbox);
     remote->server =
 	LIBBALSA_SERVER(libbalsa_server_new(LIBBALSA_SERVER_POP3));
 }
 
 static void
-libbalsa_mailbox_pop3_destroy(GtkObject * object)
+libbalsa_mailbox_pop3_finalize(GObject * object)
 {
     LibBalsaMailboxPop3 *mailbox = LIBBALSA_MAILBOX_POP3(object);
     LibBalsaMailboxRemote *remote = LIBBALSA_MAILBOX_REMOTE(object);
@@ -138,20 +141,31 @@ libbalsa_mailbox_pop3_destroy(GtkObject * object)
 
     g_free(mailbox->last_popped_uid);
 
-    gtk_object_destroy(GTK_OBJECT(remote->server));
+    g_object_unref(G_OBJECT(remote->server));
 
-    if (GTK_OBJECT_CLASS(parent_class)->destroy)
-	(*GTK_OBJECT_CLASS(parent_class)->destroy) (GTK_OBJECT(object));
+    if (G_OBJECT_CLASS(parent_class)->finalize)
+	G_OBJECT_CLASS(parent_class)->finalize(object);
 }
 
-GtkObject *
+GObject *
 libbalsa_mailbox_pop3_new(void)
 {
     LibBalsaMailbox *mailbox;
 
-    mailbox = gtk_type_new(LIBBALSA_TYPE_MAILBOX_POP3);
+    mailbox = g_object_new(LIBBALSA_TYPE_MAILBOX_POP3, NULL);
 
-    return GTK_OBJECT(mailbox);
+    return G_OBJECT(mailbox);
+}
+
+
+static void
+libbalsa_mailbox_pop3_config_changed(LibBalsaMailboxPop3* mailbox)
+{
+    g_return_if_fail(mailbox != NULL);
+    g_return_if_fail(LIBBALSA_IS_MAILBOX_POP3(mailbox));
+
+    g_signal_emit(G_OBJECT(mailbox), 
+                  libbalsa_mailbox_pop3_signals[CONFIG_CHANGED], 0);
 }
 
 static gboolean
@@ -193,8 +207,8 @@ libbalsa_mailbox_pop3_open(LibBalsaMailbox * mailbox)
 	mailbox->open_ref++;
 
 #ifdef DEBUG
-	g_print(_("LibBalsaMailboxPop3: Opening %s Refcount: %d\n"),
-		mailbox->name, mailbox->open_ref);
+	g_print(_("%s: Opening %s Refcount: %d\n"),
+		"LibBalsaMailboxPop3", mailbox->name, mailbox->open_ref);
 #endif
 
     }
@@ -204,7 +218,8 @@ libbalsa_mailbox_pop3_open(LibBalsaMailbox * mailbox)
 }
 
 /* libbalsa_mailbox_pop3_check:
-   checks=downloads POP3 mail. 
+   checks=downloads POP3 mail.
+   LOCKING : assumes gdk lock HELD and other locks (libmutt, mailbox) NOT HELD
 */
 static void
 libbalsa_mailbox_pop3_check(LibBalsaMailbox * mailbox)
@@ -217,10 +232,7 @@ libbalsa_mailbox_pop3_check(LibBalsaMailbox * mailbox)
     LibBalsaMailboxPop3 *m = LIBBALSA_MAILBOX_POP3(mailbox);
     LibBalsaServer *server;
     gboolean remove_tmp = TRUE;
-#ifdef BALSA_USE_THREADS
     gchar *msgbuf;
-    MailThreadMessage *threadmsg;
-#endif				/* BALSA_USE_THREADS */
     
     if (!m->check) return;
 
@@ -229,14 +241,9 @@ libbalsa_mailbox_pop3_check(LibBalsaMailbox * mailbox)
        !(server->passwd = libbalsa_server_get_password(server, mailbox)))
        return;
 
-    /* Unlock GDK - this is safe since libbalsa_error is threadsafe. */
-    gdk_threads_leave();
-        
-#ifdef BALSA_USE_THREADS
     msgbuf = g_strdup_printf("POP3: %s", mailbox->name);
-    MSGMAILTHREAD(threadmsg, MSGMAILTHREAD_SOURCE, msgbuf);
+    libbalsa_mailbox_progress_notify(mailbox, LIBBALSA_NTFY_SOURCE,0,0,msgbuf);
     g_free(msgbuf);
-#endif
 
     if(m->last_popped_uid) 
 	strncpy(uid, m->last_popped_uid, sizeof(uid));
@@ -260,9 +267,11 @@ libbalsa_mailbox_pop3_check(LibBalsaMailbox * mailbox)
     }
     close(tmp_file);
 
-    status =  LIBBALSA_MAILBOX_POP3(mailbox)->filter 
-	? libbalsa_fetch_pop_mail_filter (m, progress_cb, uid)
-	: libbalsa_fetch_pop_mail_direct (m, tmp_path, progress_cb, uid);
+    status =  m->filter 
+	? libbalsa_fetch_pop_mail_filter (m, uid, m->filter_cmd,
+                                          progress_cb, mailbox)
+	: libbalsa_fetch_pop_mail_direct (m, tmp_path, uid, 
+                                          progress_cb, mailbox);
 
     if(status != POP_OK)
 	libbalsa_information(LIBBALSA_INFORMATION_WARNING,
@@ -275,24 +284,12 @@ libbalsa_mailbox_pop3_check(LibBalsaMailbox * mailbox)
 	       uid) != 0) {
 	
 	g_free(LIBBALSA_MAILBOX_POP3(mailbox)->last_popped_uid);
-	
 	LIBBALSA_MAILBOX_POP3(mailbox)->last_popped_uid =
 	    g_strdup(uid);
 	
-#ifdef BALSA_USE_THREADS
-	threadmsg = g_new(MailThreadMessage, 1);
-	threadmsg->message_type = MSGMAILTHREAD_UPDATECONFIG;
-	threadmsg->mailbox = (void *) mailbox;
-	write(mail_thread_pipes[1], (void *) &threadmsg,
-	      sizeof(void *));
-#else
-	config_mailbox_update(mailbox);
-#endif
+        libbalsa_mailbox_pop3_config_changed(LIBBALSA_MAILBOX_POP3(mailbox));
     } 
 
-    /* Regrab the gdk lock before leaving */
-    gdk_threads_enter();
-
     tmp_mailbox = (LibBalsaMailbox*)
         libbalsa_mailbox_local_new(tmp_path, FALSE);
     if(!tmp_mailbox)  {
@@ -303,22 +300,7 @@ libbalsa_mailbox_pop3_check(LibBalsaMailbox * mailbox)
 	return;
     }	
     libbalsa_mailbox_open(tmp_mailbox);
-    if((m->inbox) && (tmp_mailbox->messages)) {
-	GSList * filters; 
-
-	 /* Load associated filters if needed */
-        if (!mailbox->filters)                                
-            config_mailbox_filters_load(mailbox);
-        filters = libbalsa_mailbox_filters_when(mailbox->filters,
-						FILTER_WHEN_INCOMING);
-	if (filters) {
-	    if (filters_prepare_to_run(filters)) {
-		libbalsa_filter_match(filters, tmp_mailbox->message_list);
-		libbalsa_filter_apply(filters);
-	    }
-	    g_slist_free(filters);
-	}
-
+    if ((m->inbox) && (tmp_mailbox->messages)) {
 	if (!libbalsa_messages_move(tmp_mailbox->message_list, m->inbox)) {    
 	    libbalsa_information(LIBBALSA_INFORMATION_WARNING,
 				 _("Error placing messages from %s on %s\n"
@@ -326,50 +308,36 @@ libbalsa_mailbox_pop3_check(LibBalsaMailbox * mailbox)
 				 mailbox->name, 
 				 LIBBALSA_MAILBOX(m->inbox)->name,
 				 tmp_path);
-            remove_tmp = FALSE;
+	    remove_tmp = FALSE;
 	}
-    } 
-    libbalsa_mailbox_close(LIBBALSA_MAILBOX(tmp_mailbox));
-    gtk_object_destroy(GTK_OBJECT(tmp_mailbox));	
+    }
+    libbalsa_mailbox_close(tmp_mailbox);
+    g_object_unref(G_OBJECT(tmp_mailbox));	
     if(remove_tmp) 
-        unlink(tmp_path);
+	unlink(tmp_path);
     g_free(tmp_path);
 }
 
 
 static void
-progress_cb(char *msg, int prog, int tot)
+progress_cb(void* mailbox, char *msg, int prog, int tot)
 {
-/* FIXME: We don't update progress in non threaded version?
- * I can't see how it used to work...
- */
-#ifdef BALSA_USE_THREADS
-    MailThreadMessage *message;
-
-    message = g_new(MailThreadMessage, 1);
-    switch(tot) {
-    case -1: message->message_type = MSGMAILTHREAD_FINISHED; break;
-    case 0:
-        if (prog == 0) {
-            message->message_type = MSGMAILTHREAD_MSGINFO;
-            break;
-        }
-    default: message->message_type = MSGMAILTHREAD_MSGINFO; break;
+    /* tot=-1 means finished */
+    if (tot==-1)
+	libbalsa_mailbox_progress_notify(LIBBALSA_MAILBOX(mailbox), 
+					 LIBBALSA_NTFY_PROGRESS, 0,
+					 1, "Finished");
+    else {
+	if (tot>0)
+	    libbalsa_mailbox_progress_notify(LIBBALSA_MAILBOX(mailbox), 
+					     LIBBALSA_NTFY_PROGRESS, prog,
+					     tot, msg);
+	libbalsa_mailbox_progress_notify(LIBBALSA_MAILBOX(mailbox),
+					 LIBBALSA_NTFY_MSGINFO, prog, tot, msg);
     }
-    memcpy(message->message_string, msg, strlen(msg) + 1);
-    message->num_bytes = prog;
-    message->tot_bytes = tot;
-
-    /* FIXME: There is potential for a timeout with 
-       * the server here, if we don't get the lock back
-       * soon enough.. But it prevents the main thread from
-       * blocking on the mutt_lock, andthe pipe filling up.
-       * This would give us a deadlock.
-     */
-    write(mail_thread_pipes[1], (void *) &message, sizeof(void *));
-#endif
 }
 
+
 static void
 libbalsa_mailbox_pop3_save_config(LibBalsaMailbox * mailbox,
 				  const gchar * prefix)
@@ -386,8 +354,10 @@ libbalsa_mailbox_pop3_save_config(LibBalsaMailbox * mailbox,
     gnome_config_set_bool("Delete", pop->delete_from_server);
     gnome_config_set_bool("Apop", pop->use_apop);
     gnome_config_set_bool("Filter", pop->filter);
-
-    gnome_config_set_string("Lastuid", pop->last_popped_uid);
+    if(pop->filter_cmd)
+        gnome_config_set_string("FilterCmd", pop->filter_cmd);
+    if(pop->last_popped_uid)
+        gnome_config_set_string("Lastuid", pop->last_popped_uid);
 
     if (LIBBALSA_MAILBOX_CLASS(parent_class)->save_config)
 	LIBBALSA_MAILBOX_CLASS(parent_class)->save_config(mailbox, prefix);
@@ -410,6 +380,10 @@ libbalsa_mailbox_pop3_load_config(LibBalsaMailbox * mailbox,
     pop->delete_from_server = gnome_config_get_bool("Delete=false");
     pop->use_apop = gnome_config_get_bool("Apop=false");
     pop->filter = gnome_config_get_bool("Filter=false");
+    pop->filter_cmd = gnome_config_get_string("FilterCmd");
+    if(pop->filter_cmd && *pop->filter_cmd == '\0') {
+	g_free(pop->filter_cmd); pop->filter_cmd = NULL;
+    }
 
     g_free(pop->last_popped_uid);
     pop->last_popped_uid = gnome_config_get_string("Lastuid");
