@@ -23,31 +23,34 @@
 #define __LIBBALSA_MESSAGE_H__
 
 #include <glib.h>
-#include <gtk/gtk.h>
 
+#include <stdio.h>
 #include <time.h>
 
-#include "libbalsa.h"
+#ifdef HAVE_GPGME
+#include "rfc3156.h"
+#endif
 
 #if ENABLE_ESMTP
 #include <auth-client.h>
 #endif
 
-/* do copy content, otherwise we get too many races conditions.
-   Most common was to grab subject and then see libmutt delete its data
-   under us (get_subject errors). The solution would be to 
-   lock data harder seem to require a major reengineering.
-*/
-#define MESSAGE_COPY_CONTENT 1
-
-#define LIBBALSA_TYPE_MESSAGE                      (libbalsa_message_get_type())
-#define LIBBALSA_MESSAGE(obj)                      (GTK_CHECK_CAST(obj, LIBBALSA_TYPE_MESSAGE, LibBalsaMessage))
-#define LIBBALSA_MESSAGE_CLASS(klass)              (GTK_CHECK_CLASS_CAST(klass, LIBBALSA_TYPE_MESSAGE, LibBalsaMessageClass))
-#define LIBBALSA_IS_MESSAGE(obj)                   (GTK_CHECK_TYPE(obj, LIBBALSA_TYPE_MESSAGE))
-#define LIBBALSA_IS_MESSAGE_CLASS(klass)           (GTK_CHECK_CLASS_TYPE(klass, LIBBALSA_TYPE_MESSAGE))
+/* #define MESSAGE_COPY_CONTENT 1 */
+#define LIBBALSA_TYPE_MESSAGE \
+    (libbalsa_message_get_type())
+#define LIBBALSA_MESSAGE(obj) \
+    (G_TYPE_CHECK_INSTANCE_CAST(obj, LIBBALSA_TYPE_MESSAGE, LibBalsaMessage))
+#define LIBBALSA_MESSAGE_CLASS(klass) \
+    (G_TYPE_CHECK_CLASS_CAST(klass, LIBBALSA_TYPE_MESSAGE, \
+                             LibBalsaMessageClass))
+#define LIBBALSA_IS_MESSAGE(obj) \
+    (G_TYPE_CHECK_INSTANCE_TYPE(obj, LIBBALSA_TYPE_MESSAGE))
+#define LIBBALSA_IS_MESSAGE_CLASS(klass) \
+    (G_TYPE_CHECK_CLASS_TYPE(klass, LIBBALSA_TYPE_MESSAGE))
 
 typedef struct _LibBalsaMessageClass LibBalsaMessageClass;
 typedef enum _LibBalsaMessageFlag LibBalsaMessageFlag;
+typedef enum _LibBalsaMsgCreateResult LibBalsaMsgCreateResult;
 
 enum _LibBalsaMessageFlag {
     LIBBALSA_MESSAGE_FLAG_NEW = 1 << 1,
@@ -56,40 +59,46 @@ enum _LibBalsaMessageFlag {
     LIBBALSA_MESSAGE_FLAG_FLAGGED = 1 << 4
 };
 
-struct _LibBalsaMessage {
-    GtkObject object;
+enum _LibBalsaMsgCreateResult {
+    LIBBALSA_MESSAGE_CREATE_OK,
+#ifdef HAVE_GPGME
+    LIBBALSA_MESSAGE_SIGN_ERROR,
+    LIBBALSA_MESSAGE_ENCRYPT_ERROR,
+#endif
+    LIBBALSA_MESSAGE_CREATE_ERROR,
+    LIBBALSA_MESSAGE_SEND_ERROR
+};
 
-    /* the mailbox this message belongs to */
-    LibBalsaMailbox *mailbox;
+#ifdef HAVE_GPGME
+typedef enum _LibBalsaMsgProtectState LibBalsaMsgProtectState;
 
-    /* flags */
-    LibBalsaMessageFlag flags;
-
-    MuttHeader* header;
-    /* remail header if any */
-    gchar *remail;
+enum _LibBalsaMsgProtectState {
+    LIBBALSA_MSG_PROTECT_NONE,
+    LIBBALSA_MSG_PROTECT_SIGN_UNKNOWN,
+    LIBBALSA_MSG_PROTECT_SIGN_GOOD,
+    LIBBALSA_MSG_PROTECT_SIGN_NOTRUST,
+    LIBBALSA_MSG_PROTECT_SIGN_BAD,
+    LIBBALSA_MSG_PROTECT_CRYPT
+};
+#endif
 
+/*
+ * LibBalsaMessageHeaders contains all headers which are used to display both
+ * a "main" message's headers as well as those from an embedded message/rfc822
+ * part.
+ */
+struct _LibBalsaMessageHeaders {
     /* message composition date */
     time_t date;
 
-    /* from, sender, reply, and disposition notification addresses */
+    /* subject (for embedded messages only) */
+    gchar *subject;
+
+    /* from, reply, and disposition notification addresses */
     LibBalsaAddress *from;
-    LibBalsaAddress *sender;
     LibBalsaAddress *reply_to;
     LibBalsaAddress *dispnotify_to;
 
-    /* subject line; we still need it here for sending;
-     * although _SET_SUBJECT might resolve it(?) 
-     * but we can set to to NULL unless there is no mailbox, like
-     * on sending. */
-    gchar *subj;
-#ifdef MESSAGE_COPY_CONTENT
-#define LIBBALSA_MESSAGE_GET_SUBJECT(m) ((m)->subj)
-#else
-#define LIBBALSA_MESSAGE_GET_SUBJECT(m) libbalsa_message_get_subject(m)
-#endif
-#define LIBBALSA_MESSAGE_SET_SUBJECT(m,s) \
-        { g_free((m)->subj); (m)->subj = (s); }
     /* primary, secondary, and blind recipent lists */
     GList *to_list;
     GList *cc_list;
@@ -98,6 +107,48 @@ struct _LibBalsaMessage {
     /* File Carbon Copy Mailbox URL */
     gchar *fcc_url;
 
+    /* other headers */
+    GList *user_hdrs;
+};
+
+/** FREE_HEADER_LIST() frees user_hdrs */
+#define FREE_HEADER_LIST(l) do{ g_list_foreach((l),(GFunc)g_strfreev,NULL);\
+                                g_list_free(l); } while(0)
+
+struct _LibBalsaMessage {
+    GObject object;
+
+    /* the mailbox this message belongs to */
+    LibBalsaMailbox *mailbox;
+
+    /* flags */
+    LibBalsaMessageFlag flags;
+
+    /* headers */
+    MuttHeader* header;
+    LibBalsaMessageHeaders *headers;
+    int updated; /** whether complete headers have been fetched */
+
+    /* remail header if any */
+    gchar *remail;
+
+    /* sender address */
+    LibBalsaAddress *sender;
+
+    /* subject line; we still need it here for sending;
+     * although _SET_SUBJECT might resolve it(?) 
+     * but we can set to to NULL unless there is no mailbox, like
+     * on sending. */
+    gchar *subj;
+#ifdef MESSAGE_COPY_CONTENT
+#define LIBBALSA_MESSAGE_GET_SUBJECT(m) \
+    ((m)->subj ? (m)->subj : _("(No subject)"))
+#else
+#define LIBBALSA_MESSAGE_GET_SUBJECT(m) libbalsa_message_get_subject(m)
+#endif
+#define LIBBALSA_MESSAGE_SET_SUBJECT(m,s) \
+        { g_free((m)->subj); (m)->subj = (s); }
+
     /* replied message ID's */
     GList *references;
     GList *references_for_threading; /* oldest first */
@@ -105,12 +156,23 @@ struct _LibBalsaMessage {
     /* replied message ID; from address on date */
     gchar *in_reply_to;
 
-    /* miscellaneous headers */
-    GList *user_headers;
-    
     /* message ID */
     gchar *message_id;
 
+#ifdef HAVE_GPGME
+    /* GPG sign and/or encrypt message (sending) */
+    guint gpg_mode;
+
+    /* protection (i.e. sign/encrypt) status (received message) */
+    LibBalsaMsgProtectState prot_state;
+#endif
+
+    /* a forced multipart subtype or NULL for mixed */
+    gchar *subtype;
+
+    /* additional message content type parameters */
+    GList *parameters;
+
     /* message body */
     guint body_ref;
     LibBalsaMessageBody *body_list;
@@ -129,27 +191,43 @@ struct _LibBalsaMessage {
 #define LIBBALSA_MESSAGE_GET_NO(m)  libbalsa_message_get_no(m)
 #endif
 };
-#define LIBBALSA_MESSAGE_IS_FLAGGED(m)\
- ((m)->flags&LIBBALSA_MESSAGE_FLAG_FLAGGED)
+
+#define LIBBALSA_MESSAGE_HAS_FLAG(message, mask) \
+    ((LIBBALSA_MESSAGE(message)->flags & mask) != 0)
+#define LIBBALSA_MESSAGE_IS_UNREAD(message) \
+    LIBBALSA_MESSAGE_HAS_FLAG(message, LIBBALSA_MESSAGE_FLAG_NEW)
+#define LIBBALSA_MESSAGE_IS_DELETED(message) \
+    LIBBALSA_MESSAGE_HAS_FLAG(message, LIBBALSA_MESSAGE_FLAG_DELETED)
+#define LIBBALSA_MESSAGE_IS_REPLIED(message) \
+    LIBBALSA_MESSAGE_HAS_FLAG(message, LIBBALSA_MESSAGE_FLAG_REPLIED)
+#define LIBBALSA_MESSAGE_IS_FLAGGED(message) \
+    LIBBALSA_MESSAGE_HAS_FLAG(message, LIBBALSA_MESSAGE_FLAG_FLAGGED)
+
+#define LIBBALSA_MESSAGE_SET_FLAGS(message, mask) \
+    LIBBALSA_MESSAGE(message)->flags |= mask
+#define LIBBALSA_MESSAGE_UNSET_FLAGS(message, mask) \
+    LIBBALSA_MESSAGE(message)->flags &= ~mask
 
 struct _LibBalsaMessageClass {
-    GtkObjectClass parent_class;
+    GObjectClass parent_class;
 
     /* deal with flags being set/unset */
     /* Signal: */
     void (*status_changed) (LibBalsaMessage * message,
 			    LibBalsaMessageFlag flag, gboolean);
-
-    /* Virtual Functions: */
-    void (*clear_flags) (LibBalsaMessage * message);
-    void (*set_answered) (LibBalsaMessage * message, gboolean set);
-    void (*set_read) (LibBalsaMessage * message, gboolean set);
-    void (*set_deleted) (LibBalsaMessage * message, gboolean set);
-    void (*set_flagged) (LibBalsaMessage * message, gboolean set);
 };
 
 
-GtkType libbalsa_message_get_type(void);
+GType libbalsa_message_get_type(void);
+
+/*
+ * message headers
+ */
+void libbalsa_message_headers_destroy(LibBalsaMessageHeaders * headers);
+void libbalsa_message_headers_from_mutt(LibBalsaMessageHeaders *headers,
+					MuttHeader *mheader);
+GList *libbalsa_message_user_hdrs_from_mutt(MuttHeader* header);
+
 
 /*
  * messages
@@ -160,50 +238,41 @@ gboolean libbalsa_message_copy(LibBalsaMessage * message,
 			       LibBalsaMailbox * dest);
 gboolean libbalsa_message_save(LibBalsaMessage * message,
 			       const gchar *filename);
-gboolean libbalsa_message_move(LibBalsaMessage * message,
-			       LibBalsaMailbox * mailbox);
 gboolean libbalsa_messages_move(GList * messages,
 				LibBalsaMailbox * dest);
 gboolean libbalsa_messages_copy(GList * messages,
                                 LibBalsaMailbox * dest);
-void libbalsa_message_clear_flags(LibBalsaMessage * message);
 
-void libbalsa_message_delete(LibBalsaMessage * message, gboolean del);
-void libbalsa_messages_delete(GList * messages);
+void libbalsa_messages_delete(GList * messages, gboolean del);
 
-void libbalsa_message_read(LibBalsaMessage * message, gboolean read);
-void libbalsa_message_flag(LibBalsaMessage * message, gboolean flag);
+void libbalsa_messages_read(GList * messages, gboolean read);
+void libbalsa_messages_flag(GList * messages, gboolean flag);
 
-void libbalsa_message_answer(LibBalsaMessage * message);
 void libbalsa_message_reply(LibBalsaMessage * message);
 
 void libbalsa_message_append_part(LibBalsaMessage * message,
 				  LibBalsaMessageBody * body);
 
-gboolean libbalsa_message_body_ref(LibBalsaMessage * message);
+gboolean libbalsa_message_body_ref(LibBalsaMessage * message, gboolean read);
 void libbalsa_message_body_unref(LibBalsaMessage * message);
 
-gboolean balsa_send_message(LibBalsaMessage * message,
-			    LibBalsaMailbox * outbox, gint encoding);
-gboolean balsa_postpone_message(LibBalsaMessage * message,
-				LibBalsaMailbox * draftbox,
-				LibBalsaMessage * reply_message,
-				gchar * fcc);
-void libbalsa_message_queue(LibBalsaMessage* message, 
-			    LibBalsaMailbox* outbox, LibBalsaMailbox* fccbox,
-			    gint encoding, gboolean flow);
+LibBalsaMsgCreateResult libbalsa_message_queue(LibBalsaMessage* message, 
+					       LibBalsaMailbox* outbox, LibBalsaMailbox* fccbox,
+					       gint encoding, gboolean flow);
 #if ENABLE_ESMTP
-gboolean libbalsa_message_send(LibBalsaMessage* message,
-			       LibBalsaMailbox* outbox,  
-			       LibBalsaMailbox* fccbox,
-			       gint encoding, gchar* smtp_server,
-			       auth_context_t smtp_authctx,
-			       gint tls_mode, gboolean flow);
+LibBalsaMsgCreateResult libbalsa_message_send(LibBalsaMessage* message,
+					      LibBalsaMailbox* outbox,  
+					      LibBalsaMailbox* fccbox,
+					      gint encoding, gchar* smtp_server,
+					      auth_context_t smtp_authctx,
+					      gint tls_mode, gboolean flow,
+                                              gboolean debug);
 #else
-gboolean libbalsa_message_send(LibBalsaMessage* message,
-			       LibBalsaMailbox* outbox,  
-			       LibBalsaMailbox* fccbox,
-			       gint encoding, gboolean flow);
+LibBalsaMsgCreateResult libbalsa_message_send(LibBalsaMessage* message,
+					      LibBalsaMailbox* outbox,  
+					      LibBalsaMailbox* fccbox,
+					      gint encoding, gboolean flow,
+                                              gboolean debug);
 #endif
 gboolean libbalsa_message_postpone(LibBalsaMessage * message,
 				   LibBalsaMailbox * draftbox,
@@ -214,8 +283,10 @@ gboolean libbalsa_message_postpone(LibBalsaMessage * message,
 /*
  * misc message releated functions
  */
-gchar *libbalsa_message_date_to_gchar(LibBalsaMessage * message,
-				      const gchar * date_string);
+gchar *libbalsa_message_headers_date_to_gchar(LibBalsaMessageHeaders * headers,
+					      const gchar * date_string);
+#define libbalsa_message_date_to_gchar(m,s) \
+        libbalsa_message_headers_date_to_gchar((m)->headers,s)
 gchar *libbalsa_message_size_to_gchar(LibBalsaMessage * message,
                                       gboolean lines);
 gchar *libbalsa_message_title(LibBalsaMessage * message,
@@ -223,16 +294,18 @@ gchar *libbalsa_message_title(LibBalsaMessage * message,
 gchar **libbalsa_create_hdr_pair(const gchar * name, gchar * value);
 
 const gchar *libbalsa_message_pathname(LibBalsaMessage * message);
-const gchar *libbalsa_message_charset(LibBalsaMessage * message);
+gchar *libbalsa_message_charset(LibBalsaMessage * message);
 const gchar *libbalsa_message_body_charset(LibBalsaMessageBody * body);
 gboolean libbalsa_message_is_multipart(LibBalsaMessage * message);
 gboolean libbalsa_message_has_attachment(LibBalsaMessage * message);
+#ifdef HAVE_GPGME
+gboolean libbalsa_message_is_pgp_signed(LibBalsaMessage * message);
+gboolean libbalsa_message_is_pgp_encrypted(LibBalsaMessage * message);
+#endif
 
-GList *libbalsa_message_user_hdrs(LibBalsaMessage * message);
 GList *libbalsa_message_find_user_hdr(LibBalsaMessage * message, 
                                       const gchar * find);
-LibBalsaMessage *libbalsa_message_find_by_message_id(LibBalsaMailbox * mailbox,
-                                                     gchar * msgid);
+FILE* libbalsa_message_get_part_by_id(LibBalsaMessage* msg, const gchar* id);
 
 void libbalsa_message_set_dispnotify(LibBalsaMessage *message, 
 				     LibBalsaAddress *address);
