@@ -27,12 +27,13 @@
 
 #ifdef __cplusplus
 extern "C" {
-#endif				/* __cplusplus */
+#endif                          /* __cplusplus */
 
 
-#define BALSA_MESSAGE(obj)          GTK_CHECK_CAST (obj, balsa_message_get_type (), BalsaMessage)
-#define BALSA_MESSAGE_CLASS(klass)  GTK_CHECK_CLASS_CAST (klass, balsa_message_get_type (), BalsaMessageClass)
-#define BALSA_IS_MESSAGE(obj)       GTK_CHECK_TYPE (obj, balsa_message_get_type ())
+#define BALSA_TYPE_MESSAGE          (balsa_message_get_type ())
+#define BALSA_MESSAGE(obj)          GTK_CHECK_CAST (obj, BALSA_TYPE_MESSAGE, BalsaMessage)
+#define BALSA_MESSAGE_CLASS(klass)  GTK_CHECK_CLASS_CAST (klass, BALSA_TYPE_MESSAGE, BalsaMessageClass)
+#define BALSA_IS_MESSAGE(obj)       GTK_CHECK_TYPE (obj, BALSA_TYPE_MESSAGE)
 
 
 typedef struct _BalsaMessage BalsaMessage;
@@ -41,61 +42,70 @@ typedef struct _BalsaMessageClass BalsaMessageClass;
 typedef struct _BalsaPartInfo BalsaPartInfo;
 
 struct _BalsaMessage {
-	GtkViewport parent;
+        GtkViewport parent;
 
-	/* The table widget */
-	GtkWidget *table;
+       /* The vbox widget */
+       GtkWidget *vbox;
+ 
+        /* Widget to hold headers */
+        GtkWidget *header_box;
+        GtkWidget *header_text;
+        GtkWidget *header_container;
+        ShownHeaders shown_headers;
+        gboolean show_all_headers;
 
-	/* Widget to hold headers */
-	GtkWidget *header_text;
-	ShownHeaders shown_headers;
+        /* Notebook to hold content + structure */
+        GtkWidget *notebook;
 
-	/* Widget to hold content */
-	GtkWidget *content;
-	gboolean content_has_focus;
+        /* Widgets to hold content */
+        GtkWidget *cont_viewport;
+        GtkWidget *content;
+        gboolean content_has_focus;
 
-	/* Widget to hold icons */
-	GtkWidget *part_list;
-	gint part_count;
+        /* Widget to hold structure tree */
+        GtkWidget *treeview;
+        gint info_count;
+        GList *save_all_list;
+        GtkWidget *save_all_popup;
+    
+        gboolean wrap_text;
 
-	gboolean wrap_text;
+        BalsaPartInfo *current_part;
 
-	BalsaPartInfo *current_part;
-
-	LibBalsaMessage *message;
+        LibBalsaMessage *message;
 };
 
 struct _BalsaMessageClass {
-	GtkViewportClass parent_class;
+        GtkViewportClass parent_class;
 
-	void (*select_part) (BalsaMessage * message);
+        void (*select_part) (BalsaMessage * message);
 };
 
-guint balsa_message_get_type(void);
+GtkType balsa_message_get_type(void);
 GtkWidget *balsa_message_new(void);
 
-void balsa_message_clear(BalsaMessage * bmessage);
 gboolean balsa_message_set(BalsaMessage * bmessage,
-			   LibBalsaMessage * message);
+                           LibBalsaMessage * message);
 
 void balsa_message_next_part(BalsaMessage * bmessage);
 void balsa_message_previous_part(BalsaMessage * bmessage);
 void balsa_message_save_current_part(BalsaMessage * bmessage);
 
 void balsa_message_set_displayed_headers(BalsaMessage * bmessage,
-					     ShownHeaders sh);
+                                             ShownHeaders sh);
 void balsa_message_set_wrap(BalsaMessage * bmessage, gboolean wrap);
 
 gboolean balsa_message_can_select(BalsaMessage * bmessage);
-void balsa_message_copy_clipboard(BalsaMessage * bmessage);
-void balsa_message_select_all(BalsaMessage * bmessage);
+gboolean balsa_message_grab_focus(BalsaMessage * bmessage);
 
 void reflow_string(gchar * str, gint mode, gint * cur_pos, int width);
-
-/* a helper functions; FIXME: find more proper location for them.  */
-    GdkFont* balsa_get_font_by_charset(const gchar* base, const gchar*charset);
+#ifdef HAVE_GTKHTML
+#define BALSA_MESSAGE_ZOOM_KEY "balsa-message-zoom"
+gboolean balsa_message_can_zoom(BalsaMessage * bm);
+void balsa_message_zoom(BalsaMessage * bm, gint in_out);
+#endif                          /* HAVE_GTKHTML */
 
 #ifdef __cplusplus
 }
-#endif				/* __cplusplus */
-#endif				/* __BALSA_MESSAGE_H__ */
+#endif                          /* __cplusplus */
+#endif                          /* __BALSA_MESSAGE_H__ */
