@@ -30,7 +30,11 @@ enum _BalsaInformationShow {
     BALSA_INFORMATION_SHOW_STDERR,
 };
 
-void balsa_information(LibBalsaInformationType type, GtkWindow* parent,
-                       const char *fmt, ...);
+void balsa_information(LibBalsaInformationType type, const char *fmt, ...);
+void balsa_information_parented(GtkWindow *widget,
+                                LibBalsaInformationType type, 
+                                const char *fmt, ...);
+void balsa_information_real(GtkWindow *parent, LibBalsaInformationType type,
+                            const char *msg);
 
 #endif
