@@ -22,6 +22,8 @@
 #ifndef __MAIN_WINDOW_H__
 #define __MAIN_WINDOW_H__
 
+#include "toolbar-factory.h"
+
 #define BALSA_TYPE_WINDOW		       (balsa_window_get_type ())
 #define BALSA_WINDOW(obj)		       (GTK_CHECK_CAST (obj, BALSA_TYPE_WINDOW, BalsaWindow))
 #define BALSA_WINDOW_CLASS(klass)	       (GTK_CHECK_CLASS_CAST (klass, BALSA_TYPE_WINDOW, BalsaWindowClass))
@@ -46,6 +48,8 @@ struct _BalsaWindow {
     GtkWidget *preview;		/* message is child */
     GtkWidget *hpaned;
     GtkWidget *vpaned;
+    GtkWidget *current_index;
+    LibBalsaMessage *current_message;
 };
 
 struct _BalsaWindowClass {
@@ -73,15 +77,14 @@ void balsa_window_open_mbnode(BalsaWindow * window, BalsaMailboxNode*mbnode);
 void balsa_window_close_mbnode(BalsaWindow * window, BalsaMailboxNode*mbnode);
 void enable_empty_trash(TrashState status);
 void balsa_window_enable_continue(void);
-void balsa_window_set_threading_menu(int);
 void balsa_change_window_layout(BalsaWindow *window);
 gboolean mail_progress_notify_cb(void);
 gboolean send_progress_notify_cb(void);
-gint check_new_messages_auto_cb(gpointer data);
 void check_new_messages_cb(GtkWidget *, gpointer data);
 void check_new_messages_real(GtkWidget *, gpointer data, int type);
 void empty_trash(void);
 void update_view_menu(void);
+BalsaToolbarModel *balsa_window_get_toolbar_model(void);
 
 /* functions to manipulate the progress bars of the window */
 void balsa_window_increase_activity(BalsaWindow* window);
